@@ -26,6 +26,7 @@
 #include "RasPDU.h"
 #include "RasSrv.h"
 #include "ProxyChannel.h"
+#include "h323util.h"
 #include "sigmsg.h"
 #include "cisco.h"
 #include "GkClient.h"
@@ -73,7 +74,7 @@ void AlternateGKs::Set(const H225_ArrayOf_AlternateGK & agk)
 
 bool AlternateGKs::Get(PIPSocket::Address & gkaddr, WORD & gkport)
 { 
-	if (AltGKs.size() > 0) {
+	if (!AltGKs.empty()) {
 		if (index == AltGKs.end()) {
 			index = AltGKs.begin();
 			// switch back to original GK
@@ -234,7 +235,10 @@ bool GRQRequester::SendRequest(const Address & addr, WORD pt, int r)
 	for (std::vector<Address>::iterator i = GKHome.begin(); i != GKHome.end(); ++i) {
 		if ((IsLoopback(addr) || IsLoopback(*i)) && addr != *i)
 			continue;
-		RasListener *socket = m_rasSrv->SelectInterface(*i)->GetRasListener();
+		GkInterface * inter = m_rasSrv->SelectInterface(*i);
+		if (inter == NULL)
+			return false;
+		RasListener *socket = inter->GetRasListener();
 		grq.m_rasAddress = socket->GetRasAddress(addr == INADDR_BROADCAST ? *i : addr);
 		if (addr == INADDR_BROADCAST)
 			socket->SetOption(SO_BROADCAST, 1);
@@ -337,6 +341,9 @@ GkClient::GkClient()
 
 GkClient::~GkClient()
 {
+#ifdef OpenH323Factory
+    delete m_h235Authenticators;
+#endif
 	DeleteObjectsInArray(m_handlers, m_handlers + 4);
 	delete m_gkList;
 	delete m_rewriteInfo;
@@ -448,7 +455,7 @@ bool GkClient::OnSendingRRQ(H225_RegistrationRequest &rrq)
 	return true;
 }
 
-bool GkClient::OnSendingARQ(H225_AdmissionRequest &arq, Routing::AdmissionRequest &req)
+bool GkClient::OnSendingARQ(H225_AdmissionRequest &arq, Routing::AdmissionRequest & /* req */)
 {
 	if (m_parentVendor == ParentVendor_Cisco) {
 		Cisco_ARQnonStandardInfo nonStandardData;
@@ -912,7 +919,7 @@ bool GkClient::GetAltGK()
 		// increase our resent time to avoid flooding the parent
 		m_timer = (m_resend *= 2) * 1000;
 		if (m_resend >= m_gkfailtime) {
-			// FIXME: not thread-safed
+			// TODO: not thread-safed
 			Toolkit::Instance()->GetRouteTable()->InitTable();
 			m_resend = m_gkfailtime;
 		}
@@ -935,7 +942,7 @@ void GkClient::BuildFullRRQ(H225_RegistrationRequest & rrq)
 {
 	rrq.m_terminalType.IncludeOptionalField(H225_EndpointType::e_gatekeeper);
 
-	PINDEX as, p;
+	PINDEX as;
 	if (m_endpointType == EndpointType_Terminal) {
 		rrq.m_terminalType.IncludeOptionalField(H225_EndpointType::e_terminal);
 	} else {
@@ -958,12 +965,12 @@ void GkClient::BuildFullRRQ(H225_RegistrationRequest & rrq)
 	rrq.IncludeOptionalField(H225_RegistrationRequest::e_terminalAlias);
 	as = m_h323Id.GetSize();
 	rrq.m_terminalAlias.SetSize(as);
-	for (p = 0; p < as; ++p)
+	for (PINDEX p = 0; p < as; ++p)
 		H323SetAliasAddress(m_h323Id[p], rrq.m_terminalAlias[p], H225_AliasAddress::e_h323_ID);
 
 	PINDEX s = m_e164.GetSize() + as;
 	rrq.m_terminalAlias.SetSize(s);
-	for (p = as; p < s; ++p)
+	for (PINDEX p = as; p < s; ++p)
 		H323SetAliasAddress(m_e164[p-as], rrq.m_terminalAlias[p]);
 
 	int ttl = GkConfig()->GetInteger(EndpointSection, "TimeToLive", DEFAULT_TTL);
