@@ -10,16 +10,15 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef RASTBL_H
-#define RASTBL_H "@(#) $Id: RasTbl.h,v 1.116.2.1 2007/04/26 19:29:48 willamowius Exp $"
+#define RASTBL_H "@(#) $Id: RasTbl.h,v 1.139 2008/03/06 23:47:58 willamowius Exp $"
 
 #include <list>
-#include <vector>
+#include <map>
 #include <string>
 #include "rwlock.h"
 #include "singleton.h"
 #include "h225.h"
 #include "sigmsg.h"
-#include "h323util.h"
 #include "pwlib_compat.h"
 
 
@@ -32,14 +31,10 @@ namespace Routing {
 	struct Route;
 }
 
-class GkDestAnalysisList;
 class USocket;
 class CallSignalSocket;
 class RasServer;
 class Q931;
-class SignalingMsg;
-template <class> class H225SignalingMsg;
-typedef H225SignalingMsg<H225_Setup_UUIE> SetupMsg;
 
 // Template of smart pointer
 // The class T must have Lock() & Unlock() methods
@@ -89,6 +84,7 @@ public:
 	H225_EndpointIdentifier GetEndpointIdentifier() const;
 	H225_ArrayOf_AliasAddress GetAliases() const;
 	H225_EndpointType GetEndpointType() const;
+    bool GetEndpointInfo(PString & vendor, PString & version) const;
 	int GetTimeToLive() const;
 	PIPSocket::Address GetNATIP() const;
 	CallSignalSocket *GetSocket();
@@ -96,20 +92,13 @@ public:
 	int GetCallTypeOfNumber(bool called = true) const { return called ? m_calledTypeOfNumber : m_callingTypeOfNumber; }
 	int GetProxyType() const { return m_proxy; }
 
-	/** checks if the given aliases are prefixes of the aliases which are stored
-	    for the endpoint in the registration table. #fullMatch# returns #TRUE# if
-	    a full match is found.
-	    @returns #TRUE# if a match is found
-	 */
-    bool PrefixMatch_IncompleteAddress(const H225_ArrayOf_AliasAddress &aliases,
-	                                bool &fullMatch) const;
-
 	virtual void SetRasAddress(const H225_TransportAddress &);
 	virtual void SetCallSignalAddress(const H225_TransportAddress &);
 	virtual void SetEndpointIdentifier(const H225_EndpointIdentifier &);
 	virtual void SetTimeToLive(int);
 	virtual void SetAliases(const H225_ArrayOf_AliasAddress &);
 	virtual void SetEndpointType(const H225_EndpointType &);
+    virtual void SetEndpointInfo(const PString & vendor, const PString & version);
 
 	virtual void Update(const H225_RasMessage & lightweightRRQ);
 	virtual bool IsGateway() const { return false; }
@@ -137,6 +126,12 @@ public:
 		int& matchedalias
 		) const;
 
+	virtual void LoadAliases(
+		/// aliases to be matched (one of them)
+	    const H225_ArrayOf_AliasAddress& aliases,
+		const H225_EndpointType & type
+		);
+
 	/** Load additional endpoint settings from the config file.
 	    Derived classes should call LoadConfig method of their base class
 	    at the beginning of the overriden LoadConfig.
@@ -154,6 +149,7 @@ public:
 	//virtual void BuildLCF(H225_LocationConfirm &) const;
 
 	virtual PString PrintOn(bool verbose) const;
+	PString PrintPrefixCapacities() const;
 
 	void SetNAT(bool nat);
 	void SetNATAddress(const PIPSocket::Address &);
@@ -162,6 +158,7 @@ public:
 	void SetPriority(int priority) { m_registrationPriority = priority; };
 	void SetPreemption(bool support) { m_registrationPreemption = support; };
 	void SetAssignedGatekeeper(const H225_AlternateGK & gk) { m_assignedGatekeeper = gk; };
+	bool SetAssignedAliases(H225_ArrayOf_AliasAddress & assigned);
 
 
 	/** @return
@@ -170,9 +167,10 @@ public:
 	bool IsPermanent() const;
 	bool IsUsed() const;
 	bool IsUpdated(const PTime *) const;
-	bool IsFromParent() const;
 	bool IsNATed() const;
-	bool SupportNAT() const;
+
+	H225_AlternateGK GetAssignedGatekeeper() { return m_assignedGatekeeper; }
+
 	int  Priority() const { return m_registrationPriority; }
 	bool HasNATSocket() const;
 	PTime GetUpdatedTime() const;
@@ -183,9 +181,9 @@ public:
 	 */
 	H225_RasMessage GetCompleteRegistrationRequest() const;
 
-	void AddCall();
+	void AddCall(const PString & dest);
 	void AddConnectedCall();
-	void RemoveCall();
+	void RemoveCall(const PString & dest);
 
 	void Lock();
 	void Unlock();
@@ -198,18 +196,41 @@ public:
 	    of service control sessions, if the endpoint supports call credit 
 	    capabilities.
 	*/
-	virtual void AddCallCreditServiceControl(
+	virtual bool AddCallCreditServiceControl(
 		H225_ArrayOf_ServiceControlSession& sessions, /// array to add the service control descriptor to
 		const PString& amountStr, /// user's account balance amount string
 		int billingMode, /// user's account billing mode (-1 if not set)
 		long callDurationLimit /// call duration limit (-1 if not set)
 		);
 
+	/** Append a URL related service control descriptor to the array
+	    of service control sessions
+	*/
+    virtual bool AddHTTPServiceControl(
+	    H225_ArrayOf_ServiceControlSession& sessions  /// array to add the service control descriptor to
+	    );
+
+#if H323_H350
+	/** Append a H.350 related service control descriptor to the array
+	    of service control sessions
+	*/
+    virtual bool AddH350ServiceControl(
+	    H225_ArrayOf_ServiceControlSession& sessions  /// array to add the service control descriptor to
+	    );
+#endif
+
 	/** @return
 	    True if the endpoint can handle at least one more concurrent call.
 	*/
-	bool HasAvailableCapacity() const { return m_capacity == -1 || m_activeCall < m_capacity; }
-	
+	bool HasAvailableCapacity(const H225_ArrayOf_AliasAddress & aliases) const;
+	// void DumpPrefixCapacity() const;
+	string LongestPrefixMatch(const PString & alias, int * capacity) const;
+	void UpdatePrefixStats(const PString & dest, int update);
+
+	// cause code translation
+	unsigned TranslateReceivedCause(unsigned cause) const;
+	unsigned TranslateSentCause(unsigned cause) const;
+
 	// smart pointer for EndpointRec
 	typedef SmartPtr<EndpointRec> Ptr;
 
@@ -225,6 +246,9 @@ protected:
 private:
 	/// Load general endpoint settings from the config
 	void LoadEndpointConfig();
+	void AddPrefixCapacities(const PString & prefixes);
+	void ParseTranslationMap(map<unsigned, unsigned> & cause_map, const PString & ini);
+
 
 	EndpointRec();
 	EndpointRec(const EndpointRec &);
@@ -242,9 +266,13 @@ protected:
 	H225_EndpointIdentifier m_endpointIdentifier;
 	H225_ArrayOf_AliasAddress m_terminalAliases;
 	H225_EndpointType *m_terminalType;
+	H225_VendorIdentifier *m_endpointVendor;
 	int m_timeToLive;   // seconds
 
 	int m_activeCall, m_connectedCall, m_totalCall;
+	/// active calls per prefix (regex)
+	map<string, int> m_activePrefixCalls;
+
 	int m_pollCount, m_usedCount;
 	mutable PMutex m_usedLock;
 
@@ -260,6 +288,8 @@ protected:
 	int m_callCreditSession;
 	/// endpoint call capacity, -1 means no limit
 	int m_capacity;
+	/// capacity per prefix (regex)
+	list<pair<string, int> > m_prefixCapacities;
 	int m_calledTypeOfNumber, m_callingTypeOfNumber;
 	/// Proxy Type
 	int m_proxy;
@@ -270,6 +300,9 @@ protected:
 
     /// Assigned Gatekeeper
 	H225_AlternateGK m_assignedGatekeeper;
+	/// cause code translation
+	map<unsigned, unsigned> m_receivedCauseMap;
+	map<unsigned, unsigned> m_sentCauseMap;
 };
 
 typedef EndpointRec::Ptr endptr;
@@ -384,17 +417,14 @@ public:
 	RegistrationTable();
 	~RegistrationTable();
 
-	void Initialize(GkDestAnalysisList & list) { m_destAnalysisList = &list; }
-
 	endptr InsertRec(H225_RasMessage & rrq, PIPSocket::Address = INADDR_ANY);
 	void RemoveByEndptr(const endptr & eptr);
-	void RemoveByEndpointId(const H225_EndpointIdentifier & endpointId);
 
 	endptr FindByEndpointId(const H225_EndpointIdentifier & endpointId) const;
 	endptr FindBySignalAdr(const H225_TransportAddress &, PIPSocket::Address = INADDR_ANY) const;
 	endptr FindOZEPBySignalAdr(const H225_TransportAddress &) const;
 	endptr FindByAliases(const H225_ArrayOf_AliasAddress & alias) const;
-	endptr FindEndpoint(const H225_ArrayOf_AliasAddress & alias, bool RoundRobin, bool SearchOuterZone = true);
+	endptr FindFirstEndpoint(const H225_ArrayOf_AliasAddress & alias);
 	void FindEndpoint(
 		const H225_ArrayOf_AliasAddress &aliases,
 		bool roundRobin,
@@ -405,9 +435,10 @@ public:
 	void ClearTable();
 	void CheckEndpoints();
 
-	void PrintAllRegistrations(USocket *client, BOOL verbose=FALSE);
-	void PrintAllCached(USocket *client, BOOL verbose=FALSE);
-	void PrintRemoved(USocket *client, BOOL verbose=FALSE);
+	void PrintAllRegistrations(USocket *client, bool verbose=FALSE);
+	void PrintAllCached(USocket *client, bool verbose=FALSE);
+	void PrintRemoved(USocket *client, bool verbose=FALSE);
+	void PrintPrefixCapacities(USocket *client, PString alias) const;
 
 	PString PrintStatistics() const;
 
@@ -424,7 +455,7 @@ private:
 	endptr InternalInsertOZEP(H225_RasMessage &, H225_LocationConfirm &);
 	endptr InternalInsertOZEP(H225_RasMessage &, H225_AdmissionConfirm &);
 
-	void InternalPrint(USocket *, BOOL, std::list<EndpointRec *> *, PString &);
+	void InternalPrint(USocket *, bool, std::list<EndpointRec *> *, PString &);
 	void InternalStatistics(const std::list<EndpointRec *> *, unsigned & s, unsigned & t, unsigned & g, unsigned & n) const;
 
 	void InternalRemove(iterator);
@@ -434,26 +465,24 @@ private:
 
 	template<class F> endptr InternalFind(const F & FindObject, const std::list<EndpointRec *> *ListToBeFound) const
 	{   //  The function body must be put here,
-	    //  or the Stupid VC would fail to instantiate it
+	    //  or the stupid VC would fail to instantiate it
         	ReadLock lock(listLock);
         	const_iterator Iter(find_if(ListToBeFound->begin(), ListToBeFound->end(), FindObject));
 	        return endptr((Iter != ListToBeFound->end()) ? *Iter : 0);
 	}
 
-	endptr InternalFindEP(const H225_ArrayOf_AliasAddress & alias, std::list<EndpointRec *> *ListToBeFound, bool roundrobin);
+	endptr InternalFindFirstEP(const H225_ArrayOf_AliasAddress & alias, std::list<EndpointRec *> *ListToBeFound);
 	void InternalFindEP(const H225_ArrayOf_AliasAddress & alias, std::list<EndpointRec *> *ListToBeFound, bool roundrobin, std::list<Routing::Route> &routes);
 
 	void GenerateEndpointId(H225_EndpointIdentifier &);
 	void GenerateAlias(H225_ArrayOf_AliasAddress &, const H225_EndpointIdentifier &) const;
 
-	GkDestAnalysisList & getGkDestAnalysisList() { return *m_destAnalysisList; }
 	std::list<EndpointRec *> EndpointList;
 	std::list<EndpointRec *> OuterZoneList;
 	std::list<EndpointRec *> RemovedList;
 	int regSize;
 	mutable PReadWriteMutex listLock;
 	PMutex findmutex;          // Endpoint Find Mutex
-	GkDestAnalysisList * m_destAnalysisList;
 
 	// counter to generate endpoint identifier
 	// this is NOT the count of endpoints!
@@ -469,6 +498,9 @@ private:
 template<class> class RasPDU;
 
 // record of one active call
+#ifdef hasH460
+ class H4609_QosMonitoringReportData;
+#endif
 class CallRec {
 public:
 	/// flag to overwrite proxy settings for the call
@@ -525,12 +557,10 @@ public:
 		citronNAT = 4  // caller with Citron NAT Technology?
 	};
 
-	PINDEX GetCallNumber() const
-	{ return m_CallNumber; }
-	const H225_CallIdentifier & GetCallIdentifier() const
-	{ return m_callIdentifier; }
-	const H225_ConferenceIdentifier & GetConferenceIdentifier() const
-	{ return m_conferenceIdentifier; }
+	PINDEX GetCallNumber() const { return m_CallNumber; }
+	const H225_CallIdentifier & GetCallIdentifier() const { return m_callIdentifier; }
+	unsigned GetCallRef() const { return m_crv; }
+	const H225_ConferenceIdentifier & GetConferenceIdentifier() const { return m_conferenceIdentifier; }
 	endptr GetCallingParty() const { return m_Calling; }
 	endptr GetCalledParty() const { return m_Called; }
 	endptr GetForwarder() const { return m_Forwarder; }
@@ -585,8 +615,6 @@ public:
 	void SendReleaseComplete(const H225_CallTerminationCause * = 0);
 	void BuildDRQ(H225_DisengageRequest &, unsigned reason) const;
 
-	int CountEndpoints() const;
-
 	bool CompareCallId(const H225_CallIdentifier *CallId) const;
 	bool CompareCRV(WORD crv) const;
 	bool CompareCallNumber(PINDEX CallNumber) const;
@@ -624,9 +652,15 @@ public:
 	unsigned GetDisconnectCause() const;
 
 	/** Set Q.931 ReleaseComplete cause code associated with this call. */
-	void SetDisconnectCause(
-		unsigned causeCode
-		);
+	void SetDisconnectCause(unsigned causeCode);
+
+	/** @return
+		Q.931 ReleaseComplete cause code after translation.
+	*/
+	unsigned GetDisconnectCauseTranslated() const;
+
+	/** Set Q.931 ReleaseComplete cause code after translation rules for this call. */
+	void SetDisconnectCauseTranslated(unsigned causeCode);
 
 	/// @return	Information about who disconnected the call (see #ReleaseSource enum#)
 	int GetReleaseSource() const;
@@ -641,26 +675,6 @@ public:
 		long seconds /// duration limit to be set
 		);
 
-	/** @return
-		Duration limit (in seconds) set for this call.
-		0 if call duration is not limited.
-	*/
-	long GetDurationLimit() const;
-
-	/** This function can be used to determine, if the call has been
-		disconnected due to call duration limit excess.
-
-		@return
-		true if the call duration limit has been exceeded, false otherwise.
-	*/
-	bool IsDurationLimitExceeded() const;
-
-	/** @return
-		Timestamp (number of seconds since 1st January 1970) for the call creation
-		(when this CallRec object has been instantiated).
-	*/
-	time_t GetCreationTime() const;
-
 	/** @return
 		Timestamp (number of seconds since 1st January 1970)
 		for the Setup message associated with this call. 0 if Setup
@@ -795,6 +809,8 @@ public:
 		const H225_TransportAddress & addr /// new signalling transport address
 		);
 
+	H225_TransportAddress GetSrcSignalAddr() const;
+
 	/** Get IP and port for the calling party. It is a signal address
 		for registered endpoints and remote signalling socket address
 		for unregistered endpoints.
@@ -874,7 +890,6 @@ public:
 	*/
 	PString GetCallLinkage()const { return m_callLinkage; };
 
-
 	/// Set calling party's number
 	void SetCalledStationId(
 		const PString& id /// Called-Station-Id
@@ -937,6 +952,10 @@ public:
 	void SetRADIUSClass(const PBYTEArray &bytes);
 	void SetRADIUSClass(void * bytes, PINDEX len);
 	PBYTEArray GetRADIUSClass() const;
+	
+	bool IsProceedingSent() const { return m_proceedingSent; };
+	// TODO: JW also set when sent via status port
+	void SetProceedingSent(bool val) { m_proceedingSent = val; };
 
 private:
 	void SendDRQ();
@@ -1000,6 +1019,8 @@ private:
 	long m_durationLimit;
 	/// Q.931 release complete cause code
 	unsigned m_disconnectCause;
+	/// Q.931 release complete cause code after translation
+	unsigned m_disconnectCauseTranslated;
 	/// who disconnected the call (see #RelaseSource enum#)
 	int  m_releaseSource;
 	/// unique accounting session id associated with this call
@@ -1056,11 +1077,14 @@ private:
 	PString m_codec;
 	PIPSocket::Address m_mediaOriginatingIp;
 	PBYTEArray m_radiusClass;
+	bool m_proceedingSent;
 };
 
 typedef CallRec::Ptr callptr;
 
 // all active calls
+
+class H4609_QosMonitoringReportData;
 class CallTable : public Singleton<CallTable>
 {
 public:
@@ -1093,13 +1117,17 @@ public:
 	void RemoveCall(const callptr &);
 	void RemoveFailedLeg(const callptr &);
 
-	void PrintCurrentCalls(USocket *client, BOOL verbose=FALSE) const;
+	void PrintCurrentCalls(USocket *client, bool verbose=FALSE) const;
 	PString PrintStatistics() const;
 
-	void AddForwardedCall(const callptr &);
-	endptr IsForwardedCall(const callptr &);
+#ifdef hasH460
+	void OnQosMonitoringReport(const PString &,const endptr &, H4609_QosMonitoringReportData &);
+    void QoSReport(const H225_DisengageRequest &, const endptr &, const PASN_OctetString &);
+    void QoSReport(const H225_InfoRequestResponse &, const callptr &, const endptr &, const PASN_OctetString &);
+#endif
 
 	void LoadConfig();
+	void UpdatePrefixCapacityCounters();    // after Reload
 
 	PINDEX Size() const { return m_activeCall; }
 
@@ -1134,11 +1162,10 @@ private:
 
 	bool InternalRemovePtr(CallRec *call);
 	void InternalRemove(const H225_CallIdentifier & CallId);
-	void InternalRemove(WORD CallRef);
 	void InternalRemove(iterator);
 	void InternalRemoveFailedLeg(iterator);
 
-	void InternalStatistics(unsigned & n, unsigned & act, unsigned & nb, unsigned & np, PString & msg, BOOL verbose) const;
+	void InternalStatistics(unsigned & n, unsigned & act, unsigned & nb, unsigned & np, PString & msg, bool verbose) const;
 
 	std::list<CallRec *> CallList;
 	std::list<CallRec *> RemovedList;
@@ -1221,6 +1248,33 @@ inline H225_EndpointType EndpointRec::GetEndpointType() const
 	return *m_terminalType;
 }
 
+inline bool EndpointRec::GetEndpointInfo(PString & vendor, PString & version ) const
+{
+	PWaitAndSignal lock(m_usedLock);
+
+	if (!m_endpointVendor)
+		return false;
+
+	if (m_endpointVendor->HasOptionalField(H225_VendorIdentifier::e_productId))
+		vendor = m_endpointVendor->m_productId.AsString();
+	if (m_endpointVendor->HasOptionalField(H225_VendorIdentifier::e_versionId))
+		version = m_endpointVendor->m_versionId.AsString();
+
+	return true;
+}
+
+inline void EndpointRec::SetEndpointInfo(const PString & vendor, const PString & version )
+{
+	PWaitAndSignal lock(m_usedLock);
+
+	m_endpointVendor = new H225_VendorIdentifier();
+
+	m_endpointVendor->IncludeOptionalField(H225_VendorIdentifier::e_productId);
+	   m_endpointVendor->m_productId.SetValue(vendor); 
+	m_endpointVendor->IncludeOptionalField(H225_VendorIdentifier::e_versionId);
+	   m_endpointVendor->m_versionId.SetValue(version);
+}
+
 inline void EndpointRec::SetNAT(bool nat)
 { 
 	m_nat = nat;
@@ -1236,11 +1290,6 @@ inline bool EndpointRec::IsNATed() const
 	return m_nat;
 }
 
-inline bool EndpointRec::SupportNAT() const
-{
-	return m_natsupport;
-}
-
 inline PIPSocket::Address EndpointRec::GetNATIP() const
 {
 	PWaitAndSignal lock(m_usedLock);
@@ -1272,11 +1321,6 @@ inline bool EndpointRec::IsUpdated(const PTime *now) const
 	return (!m_timeToLive || (*now - m_updatedTime).GetSeconds() < m_timeToLive);
 }
 
-inline bool EndpointRec::IsFromParent() const
-{
-	return m_fromParent;
-}
-
 inline bool EndpointRec::HasNATSocket() const
 {
 	return m_natsocket;
@@ -1299,10 +1343,13 @@ inline bool EndpointRec::HasCallCreditCapabilities() const
 	return m_hasCallCreditCapabilities;
 }
 
-inline void EndpointRec::AddCall()
+void UpdatePrefixStats(const PString & dest, int update);
+
+inline void EndpointRec::AddCall(const PString & dest)
 {
 	PWaitAndSignal lock(m_usedLock);
 	++m_activeCall, ++m_totalCall;
+	UpdatePrefixStats(dest, +1);
 }
 
 inline void EndpointRec::AddConnectedCall()
@@ -1311,10 +1358,11 @@ inline void EndpointRec::AddConnectedCall()
 	++m_connectedCall;
 }
 
-inline void EndpointRec::RemoveCall()
+inline void EndpointRec::RemoveCall(const PString & dest)
 {
 	PWaitAndSignal lock(m_usedLock);
 	--m_activeCall;
+	UpdatePrefixStats(dest, -1);
 }
 
 inline void EndpointRec::Lock()
@@ -1368,16 +1416,6 @@ inline bool CallRec::CompareSigAdr(const H225_TransportAddress *adr) const
 		(m_Called && m_Called->GetCallSignalAddress() == *adr);
 }
 
-inline long CallRec::GetDurationLimit() const
-{
-	return m_durationLimit;
-}
-
-inline time_t CallRec::GetCreationTime() const
-{
-	return m_creationTime;
-}
-
 inline time_t CallRec::GetSetupTime() const
 {
 	return m_setupTime;
@@ -1410,6 +1448,16 @@ inline void CallRec::SetDisconnectCause( unsigned causeCode )
 		m_disconnectCause = causeCode;
 }
 
+inline unsigned CallRec::GetDisconnectCauseTranslated() const
+{
+	return (m_disconnectCauseTranslated == 0) ? m_disconnectCause : m_disconnectCauseTranslated;
+}
+
+inline void CallRec::SetDisconnectCauseTranslated( unsigned causeCode )
+{
+	m_disconnectCauseTranslated = causeCode;
+}
+
 inline bool CallRec::IsTimeout(const time_t now) const
 {
 	bool result = (m_timeout > 0) && (now >= m_timer) && ((now - m_timer) >= m_timeout);
@@ -1422,4 +1470,41 @@ inline bool CallRec::IsTimeout(const time_t now) const
 	return result;
 }
 
+class PreliminaryCall
+{
+public:
+	PreliminaryCall(CallSignalSocket * callerSocket, H225_CallIdentifier id, unsigned ref)
+		: m_socket(callerSocket), m_callid(id), m_callref(ref) { };
+	~PreliminaryCall() { };
+
+	CallSignalSocket * GetCallSignalSocketCalling() const { return m_socket; };
+	H225_CallIdentifier GetCallIdentifier() const { return m_callid; };
+	unsigned GetCallRef() const { return m_callref; };
+
+private:
+	CallSignalSocket * m_socket;
+	H225_CallIdentifier m_callid;
+	unsigned m_callref;
+};
+
+// hold data about calls being established, only valid during the routing process before calls are accepted or rejected
+class PreliminaryCallTable : public Singleton<PreliminaryCallTable>
+{
+public:
+	PreliminaryCallTable();
+	~PreliminaryCallTable();
+	
+	void Insert(PreliminaryCall * call);
+	void Remove(H225_CallIdentifier id);
+	PreliminaryCall * Find(H225_CallIdentifier id) const;
+
+private:
+	PreliminaryCallTable(const PreliminaryCallTable &);
+	PreliminaryCallTable& operator==(const PreliminaryCallTable &);
+
+	mutable PReadWriteMutex tableLock;
+	// cid -> call
+	map<H225_CallIdentifier, PreliminaryCall*> calls;
+};
+
 #endif // RASTBL_H
