@@ -33,6 +33,7 @@
 #include "sigmsg.h"
 #include "Routing.h"
 #include "pwlib_compat.h"
+#include "gksql.h"
 
 using std::string;
 using std::vector;
@@ -61,7 +62,7 @@ Route::Route() : m_proxyMode(CallRec::ProxyDetect), m_flags(0)
 Route::Route(
 	const endptr &destEndpoint
 	) : m_destAddr(destEndpoint->GetCallSignalAddress()), m_destEndpoint(destEndpoint),
-	m_proxyMode(CallRec::ProxyDetect), m_flags(0)
+	m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_destNumber("")
 {
 	Toolkit::Instance()->SetRerouteCauses(m_rerouteCauses);
 }
@@ -69,7 +70,7 @@ Route::Route(
 Route::Route(
 	const PString &policyName,
 	const H225_TransportAddress &destAddr
-	) : m_destAddr(destAddr), m_policy(policyName), m_proxyMode(CallRec::ProxyDetect), m_flags(0)
+	) : m_destAddr(destAddr), m_policy(policyName), m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_destNumber("")
 {
 	Toolkit::Instance()->SetRerouteCauses(m_rerouteCauses);
 }
@@ -79,7 +80,7 @@ Route::Route(
 	const PIPSocket::Address &destIpAddr,
 	WORD destPort
 	) : m_destAddr(SocketToH225TransportAddr(destIpAddr, destPort)),
-	m_policy(policyName), m_proxyMode(CallRec::ProxyDetect), m_flags(0)
+	m_policy(policyName), m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_destNumber("")
 {
 	Toolkit::Instance()->SetRerouteCauses(m_rerouteCauses);
 }
@@ -87,7 +88,7 @@ Route::Route(
 PString Route::AsString() const
 {
 	return AsDotString(m_destAddr) + " (policy: " + m_policy + ", proxy: "
-		+ PString(m_proxyMode) + ", flags: " + PString(m_flags) + ")";
+		+ PString(m_proxyMode) + ", flags: " + PString(m_flags) + ", DialedNumber: " + m_destNumber + ")";
 }
 
 bool Route::IsFailoverActive(
@@ -575,7 +576,7 @@ bool DNSPolicy::FindByAliases(
 	return false;
 }
 
-bool DNSPolicy::FindByAliases(LocationRequest & request, H225_ArrayOf_AliasAddress & aliases)
+bool DNSPolicy::FindByAliases(LocationRequest & /* request */, H225_ArrayOf_AliasAddress & /* aliases */)
 { 
 	PTRACE(4, "ROUTING\tPolicy DNS not supported for LRQ");
 	return false; // DNSPolicy::FindByAliases((RoutingRequest&)request, aliases);
@@ -596,7 +597,7 @@ VirtualQueue::~VirtualQueue()
 	int numrequests = m_pendingRequests.size();
 	if( numrequests )
 		PTRACE(1,"VQueue\tDestroying virtual queue with "
-			<<numrequests<<" pending requests"
+			<< numrequests << " pending requests"
 			);
 	RouteRequests::iterator i = m_pendingRequests.begin();
 	while (i != m_pendingRequests.end()) {
@@ -621,10 +622,10 @@ void VirtualQueue::OnReload()
 			?"RequestTimeout":"CTI_Timeout", 
 		DEFAULT_ROUTE_REQUEST_TIMEOUT
 		) * 1000;
-	m_requestTimeout = PMIN(PMAX(100,m_requestTimeout),20000);
-	
+	m_requestTimeout = PMAX(100,m_requestTimeout);	// min wait: 100 msec
+
 	m_active = false;
-	
+
 	m_virtualQueueAliases.RemoveAll();
 	PString vqueues = GkConfig()->GetString(CTIsection, "VirtualQueueAliases", "");
 	if( vqueues.IsEmpty() ) // backward compatibility
@@ -637,7 +638,7 @@ void VirtualQueue::OnReload()
 			m_active = true;
 		}
 	}
-	
+
 	m_virtualQueuePrefixes.RemoveAll();
 	vqueues = GkConfig()->GetString(CTIsection, "VirtualQueuePrefixes", "");
 	if( !vqueues.IsEmpty() ) {
@@ -679,13 +680,17 @@ bool VirtualQueue::SendRouteRequest(
 	/// destination (virtual queue) aliases as specified
 	/// by the calling endpoint (modified by this function on successful return)
 	PString* callSigAdr,
+	/// should the call be rejected modified by this function on return)
+	bool & reject,
 	/// actual virtual queue name (should be present in destinationInfo too)
 	const PString& vqueue,
 	/// a sequence of aliases for the calling endpoint
 	/// (in the "alias:type[=alias:type]..." format)
 	const PString& sourceInfo,
 	/// the callID as string
-	const PString& callID
+	const PString& callID,
+	/// the called IP for unregistered calls
+	const PString& calledip
 	)
 {
 	bool result = false;
@@ -698,11 +703,10 @@ bool VirtualQueue::SendRouteRequest(
 				(const char *)vqueue,
 				(const char *)sourceInfo
 			   );
-		if (Toolkit::AsBool(GkConfig()->GetString("Gatekeeper::Main", "SignalCallId", 0))) {
-			PString cid = callID;
-			cid.Replace(" ", "-", true);
-			msg += PString("|") + cid;
-		}
+		PString cid = callID;
+		cid.Replace(" ", "-", true);
+		msg += PString("|") + cid;
+		msg += PString("|") + calledip;
 		msg += PString(";");
 		// signal RouteRequest to the status line only once
 		if( duprequest )
@@ -712,8 +716,9 @@ bool VirtualQueue::SendRouteRequest(
 			GkStatus::Instance()->SignalStatus(msg + "\r\n", STATUS_TRACE_LEVEL_ROUTEREQ);
 		}
 
-		// wait for an answer from the status line (routetoalias,routereject)
+		// wait for an answer from the status line (routetoalias,routetogateway,routereject)
 		result = r->m_sync.Wait(m_requestTimeout);
+		reject = r->m_reject;   // set reject status
 		m_listMutex.Wait();
 		m_pendingRequests.remove(r);
 		m_listMutex.Signal();
@@ -755,7 +760,9 @@ bool VirtualQueue::RouteToAlias(
 	/// CRV of the call associated with the route request
 	unsigned crv,
 	/// callID of the call associated with the route request
-	const PString& callID
+	const PString& callID,
+	/// should this call be rejected
+	bool reject
 	)
 {
 	PWaitAndSignal lock(m_listMutex);
@@ -768,30 +775,36 @@ bool VirtualQueue::RouteToAlias(
 		bool match = ((r->m_callingEpId == callingEpId) && (r->m_crv == crv));
 		if (!r->m_callID.IsEmpty() && !callID.IsEmpty()) {
 			// backward compatibility: only check if set
-			match = match && (r->m_callID == callID);
+			match = (r->m_callID == callID);
 		}
 		if (match) {
-			// replace virtual queue aliases info with agent aliases
-			*r->m_agent = agent;
-			if (!destinationip.IsEmpty())	// RoteToGateway
-				*r->m_callsignaladdr = destinationip;
+			if (!reject) {
+				// replace virtual queue aliases info with agent aliases
+				// TODO: add alias field in ARQ, LRQ if not present, but call is routed to alias ?
+				// TODO: remove alias field if new agent field is size 0 ??
+				if (r->m_agent)
+					*(r->m_agent) = agent;
+				if (!destinationip.IsEmpty())	// RouteToGateway
+					*(r->m_callsignaladdr) = destinationip;
+			}
+			r->m_reject = reject;
 			r->m_sync.Signal();
-			if( !foundrequest ) {
+			if (!foundrequest) {
 				foundrequest = true;
-				if( agent.GetSize() > 0 )
-					PTRACE(2,"VQueue\tRoute request (EPID :"<<callingEpId
-						<<", CRV="<<crv<<") accepted by agent "<<AsString(agent));
+				if (!reject)
+					PTRACE(2,"VQueue\tRoute request (EPID:" << callingEpId
+						<< ", CRV=" << crv << ") accepted by agent " << AsString(agent));
 				else
-					PTRACE(2,"VQueue\tRoute request (EPID :"<<callingEpId
-						<<", CRV="<<crv<<") rejected");
+					PTRACE(2,"VQueue\tRoute request (EPID:" << callingEpId
+						<< ", CRV=" << crv << ") rejected");
 			}
 		}
 		++i;
 	}
 	
 	if( !foundrequest )
-		PTRACE(4,"VQueue\tPending route request (EPID:"<<callingEpId
-			<<", CRV="<<crv<<") not found - ignoring RouteToAlias / RouteToGateway command");
+		PTRACE(4, "VQueue\tPending route request (EPID:" << callingEpId
+			<< ", CRV=" << crv << ") not found - ignoring RouteToAlias / RouteToGateway command");
 	
 	return foundrequest;
 }
@@ -807,13 +820,17 @@ bool VirtualQueue::RouteToAlias(
 	/// CRV for the call associated with the route request
 	unsigned crv,
 	/// callID of the call associated with the route request
-	const PString& callID
+	const PString& callID,
+	/// should this call be rejected
+	bool reject
 	)
 {
 	H225_ArrayOf_AliasAddress alias;
-	alias.SetSize(1);
-	H323SetAliasAddress(targetAlias, alias[0]);
-	return RouteToAlias(alias, destinationIp, callingEpId, crv, callID);
+	if (targetAlias != "") {
+		alias.SetSize(1);
+		H323SetAliasAddress(targetAlias, alias[0]);
+	}
+	return RouteToAlias(alias, destinationIp, callingEpId, crv, callID, reject);
 }
 
 bool VirtualQueue::RouteReject(
@@ -826,7 +843,7 @@ bool VirtualQueue::RouteReject(
 	)
 {
 	H225_ArrayOf_AliasAddress nullAgent;
-	return RouteToAlias(nullAgent, "", callingEpId, crv, callID);
+	return RouteToAlias(nullAgent, "", callingEpId, crv, callID, true);
 }
 
 VirtualQueue::RouteRequest* VirtualQueue::InsertRequest(
@@ -880,16 +897,15 @@ class VirtualQueuePolicy : public Policy {
 public:
 	VirtualQueuePolicy();
 
-private:
+protected:
 	// override from class Policy
-	virtual bool IsActive();
+	virtual bool IsActive() const;
 
 	virtual bool OnRequest(AdmissionRequest &);
 	virtual bool OnRequest(LocationRequest &);
-	// TODO
-	//virtual bool OnRequest(SetupRequest &);
-	//virtual bool OnRequest(FacilityRequest &);
+	virtual bool OnRequest(SetupRequest &);
 
+private:
 	VirtualQueue *m_vqueue;
 };
 
@@ -899,93 +915,191 @@ VirtualQueuePolicy::VirtualQueuePolicy()
 	m_name = "VirtualQueue";
 }
 
-bool VirtualQueuePolicy::IsActive()
+bool VirtualQueuePolicy::IsActive() const
 {
 	return m_vqueue->IsActive();
 }
 
 bool VirtualQueuePolicy::OnRequest(AdmissionRequest & request)
 {
-	if (H225_ArrayOf_AliasAddress *aliases = request.GetAliases()) {
-		const PString agent(AsString((*aliases)[0], FALSE));
-		if (m_vqueue->IsDestinationVirtualQueue(agent)) {
-			H225_AdmissionRequest & arq = request.GetRequest();
-			PTRACE(5,"Routing\tPolicy "<<m_name<<" destination matched "
-				"a virtual queue "<<agent<<" (ARQ "
-				<<arq.m_requestSeqNum.GetValue()<<')'
-				);
-			endptr ep = RegistrationTable::Instance()->FindByEndpointId(arq.m_endpointIdentifier); // should not be null
-			if (ep) {
-				PString source = AsDotString(ep->GetCallSignalAddress());
-				PString epid = ep->GetEndpointIdentifier().GetValue();
-				PString * callSigAdr = new PString();
-				if (m_vqueue->SendRouteRequest(source, epid, unsigned(arq.m_callReferenceValue), aliases, callSigAdr, agent, AsString(arq.m_srcInfo), AsString(arq.m_callIdentifier.m_guid)))
-					request.SetFlag(RoutingRequest::e_aliasesChanged);
-				if (!callSigAdr->IsEmpty()) {
-					if (!arq.HasOptionalField(H225_AdmissionRequest::e_destCallSignalAddress)) {
-						arq.IncludeOptionalField(H225_AdmissionRequest::e_destCallSignalAddress);
-					}
-					PStringArray adr_parts = callSigAdr->Tokenise(":", FALSE);
-					PString ip = adr_parts[0];
-					WORD port = (WORD)(adr_parts[1].AsInteger());
-					if (port == 0)
-						port = 1720;
-					arq.m_destCallSignalAddress = SocketToH225TransportAddr(ip, port);
-				}
-				delete callSigAdr;
+	bool reject = false;
+	H225_ArrayOf_AliasAddress *aliases = NULL;
+	PString vq = "";
+	if ((aliases = request.GetAliases()))
+		vq = AsString((*aliases)[0], FALSE);
+	if (m_vqueue->IsDestinationVirtualQueue(vq)) {
+		H225_AdmissionRequest & arq = request.GetRequest();
+		PTRACE(5,"Routing\tPolicy " << m_name << " destination matched "
+			"a virtual queue " << vq << " (ARQ "
+			<< arq.m_requestSeqNum.GetValue() << ')'
+			);
+		endptr ep = RegistrationTable::Instance()->FindByEndpointId(arq.m_endpointIdentifier); // should not be null
+		if (ep) {
+			PString source = AsDotString(ep->GetCallSignalAddress());
+			PString epid = ep->GetEndpointIdentifier().GetValue();
+			PString * callSigAdr = new PString();
+			if (m_vqueue->SendRouteRequest(source, epid, unsigned(arq.m_callReferenceValue), aliases, callSigAdr, reject, vq, AsString(arq.m_srcInfo), AsString(arq.m_callIdentifier.m_guid)))
+				request.SetFlag(RoutingRequest::e_aliasesChanged);
+			if (reject) {
+				request.SetFlag(RoutingRequest::e_Reject);
 			}
-			// the trick: if empty, the request is rejected
-			// so we return true to terminate the routing
-			// decision process, otherwise the aliases is
-			// rewritten, we return false to let subsequent
-			// policies determine the request 
-			if (m_next == NULL || aliases->GetSize() == 0)
-				return true;
+			if (!callSigAdr->IsEmpty()) {
+				if (!arq.HasOptionalField(H225_AdmissionRequest::e_destCallSignalAddress)) {
+					arq.IncludeOptionalField(H225_AdmissionRequest::e_destCallSignalAddress);
+				}
+				PStringArray adr_parts = callSigAdr->Tokenise(":", FALSE);
+				PString ip = adr_parts[0];
+				WORD port = (WORD)(adr_parts[1].AsInteger());
+				if (port == 0)
+					port = GK_DEF_ENDPOINT_SIGNAL_PORT;
+				arq.m_destCallSignalAddress = SocketToH225TransportAddr(ip, port);
+			}
+			delete callSigAdr;
 		}
+		// the trick: if empty, the request is rejected
+		// so we return true to terminate the routing
+		// decision process, otherwise the aliases is
+		// rewritten, we return false to let subsequent
+		// policies determine the request 
+		if (m_next == NULL || reject)
+			return true;
 	}
 	return false;
 }
 
 bool VirtualQueuePolicy::OnRequest(LocationRequest & request)
 {
+	bool reject = false;
 	if (H225_ArrayOf_AliasAddress *aliases = request.GetAliases()) {
-		const PString agent(AsString((*aliases)[0], FALSE));
-		if (m_vqueue->IsDestinationVirtualQueue(agent)) {
+		const PString vq(AsString((*aliases)[0], FALSE));
+		if (m_vqueue->IsDestinationVirtualQueue(vq)) {
 			H225_LocationRequest & lrq = request.GetRequest();
 			PTRACE(5,"Routing\tPolicy " << m_name << " destination matched "
-				"a virtual queue " << agent << " (LRQ "
+				"a virtual queue " << vq << " (LRQ "
 				<< lrq.m_requestSeqNum.GetValue() << ')'
 				);
 
 			// only use vqueue if sender is able to handle changed destination
-			if (lrq.HasOptionalField(H225_LocationRequest::e_canMapAlias)) {
-				if (!lrq.m_canMapAlias) {
-					PTRACE(5, "Sender can't map destination alias, skipping virtual queue");
-					return false;
-				}
+			if (!lrq.HasOptionalField(H225_LocationRequest::e_canMapAlias) || !lrq.m_canMapAlias) {
+				PTRACE(5, "Sender can't map destination alias, skipping virtual queue");
+				return false;
 			}
 
 			PString source = AsDotString(lrq.m_replyAddress);
 			PString epid = lrq.m_endpointIdentifier.GetValue();
-			if (epid.IsEmpty())
-				epid = lrq.m_gatekeeperIdentifier.GetValue() + "_" + AsString(lrq.m_sourceInfo, false);
+			if (epid.IsEmpty()) {
+				epid = lrq.m_gatekeeperIdentifier.GetValue();
+				if (lrq.HasOptionalField(H225_LocationRequest::e_sourceInfo) && (lrq.m_sourceInfo.GetSize() > 0))
+					epid += "_" + AsString(lrq.m_sourceInfo, false);
+			}
+			if (epid.IsEmpty()) {
+				epid = "unknown";	// make sure its not empty
+			}
 			PString * callSigAdr = new PString(); /* unused for LRQs */
-			PString callID = "";	/* not available for LRQs */
-			if (m_vqueue->SendRouteRequest(source, epid, unsigned(lrq.m_requestSeqNum), aliases, callSigAdr, agent, AsString(lrq.m_sourceInfo), callID))
+			PString sourceInfo = "";
+			if (lrq.HasOptionalField(H225_LocationRequest::e_sourceInfo) && (lrq.m_sourceInfo.GetSize() > 0))
+				sourceInfo = AsString(lrq.m_sourceInfo);
+			PString callID = "-";	/* not available for LRQs */
+			if (m_vqueue->SendRouteRequest(source, epid, unsigned(lrq.m_requestSeqNum), aliases, callSigAdr, reject, vq, sourceInfo, callID))
 				request.SetFlag(RoutingRequest::e_aliasesChanged);
+			if (reject) {
+				request.SetFlag(RoutingRequest::e_Reject);
+			}
+			if (!reject && !callSigAdr->IsEmpty()) {
+				// 'explicit' policy can't handle LRQs, so we do it directly
+				PStringArray adr_parts = callSigAdr->Tokenise(":", FALSE);
+				PString ip = adr_parts[0];
+				WORD port = (WORD)(adr_parts[1].AsInteger());
+				if (port == 0)
+					port = GK_DEF_ENDPOINT_SIGNAL_PORT;
+				Route route("vqueue", SocketToH225TransportAddr(ip, port));
+				route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(
+					route.m_destAddr
+				);
+				request.AddRoute(route);					
+				delete callSigAdr;
+				return true;	// stop processing
+			}
 			delete callSigAdr;
 			// the trick: if empty, the request is rejected
 			// so we return true to terminate the routing
 			// decision process, otherwise the aliases is
 			// rewritten, we return false to let subsequent
 			// policies determine the request
-			if (m_next == NULL || aliases->GetSize() == 0)
+			if (m_next == NULL || reject)
 				return true;
 		}
 	}
 	return false;
 }
 
+bool VirtualQueuePolicy::OnRequest(SetupRequest & request)
+{
+	bool reject = false;
+	H225_ArrayOf_AliasAddress *aliases = new H225_ArrayOf_AliasAddress;
+	aliases->SetSize(1);
+	PString vq = "";
+	if (request.GetAliases()) {
+		vq = AsString((*request.GetAliases())[0], false);
+	}
+	if (m_vqueue->IsDestinationVirtualQueue(vq)) {
+		H225_Setup_UUIE &setup = request.GetRequest();
+		PString callerip = AsDotString(setup.m_sourceCallSignalAddress);
+		PString epid = "unregistered";
+		const unsigned crv = request.GetWrapper()->GetCallReference();
+		PString * callSigAdr = new PString();
+		PString callid = AsString(setup.m_callIdentifier.m_guid);
+		PString src = AsString(setup.m_sourceAddress);
+		PIPSocket::Address localAddr;
+		WORD localPort;
+		request.GetWrapper()->GetLocalAddr(localAddr, localPort);
+		PString calledIP = localAddr;
+		PTRACE(5,"Routing\tPolicy " << m_name << " destination matched "
+			"a virtual queue " << vq << " (Setup "
+			<< crv << ')'
+			);
+
+		if (m_vqueue->SendRouteRequest(callerip, epid, crv, aliases, callSigAdr, reject, vq, src, callid, calledIP))
+			request.SetFlag(RoutingRequest::e_aliasesChanged);
+		
+		if (reject) {
+			request.SetFlag(RoutingRequest::e_Reject);
+		}
+		if (!reject && callSigAdr->IsEmpty()) {
+			if (!setup.HasOptionalField(H225_Setup_UUIE::e_destinationAddress)) {
+				setup.IncludeOptionalField(H225_Setup_UUIE::e_destinationAddress);
+			}
+			setup.m_destinationAddress = *aliases;
+		}
+		if (!reject && !callSigAdr->IsEmpty()) {
+			if (!setup.HasOptionalField(H225_Setup_UUIE::e_destinationAddress)) {
+				setup.IncludeOptionalField(H225_Setup_UUIE::e_destinationAddress);
+			}
+			setup.m_destinationAddress = *aliases;
+			if (!setup.HasOptionalField(H225_Setup_UUIE::e_destCallSignalAddress)) {
+				setup.IncludeOptionalField(H225_Setup_UUIE::e_destCallSignalAddress);
+			}
+			PStringArray adr_parts = callSigAdr->Tokenise(":", FALSE);
+			PString ip = adr_parts[0];
+			WORD port = (WORD)(adr_parts[1].AsInteger());
+			if (port == 0)
+				port = GK_DEF_ENDPOINT_SIGNAL_PORT;
+			setup.m_destCallSignalAddress = SocketToH225TransportAddr(ip, port);
+		}
+		delete aliases;
+		delete callSigAdr;
+		// the trick: if empty, the request is rejected
+		// so we return true to terminate the routing
+		// decision process, otherwise the aliases is
+		// rewritten, we return false to let subsequent
+		// policies determine the request
+		if (m_next == NULL || reject) {
+			return true;
+		}
+	}
+	return false;
+}
+
 class NumberAnalysisPolicy : public Policy {
 public:
 	struct PrefixEntry {
@@ -999,9 +1113,6 @@ public:
 protected:
 	virtual bool OnRequest(AdmissionRequest &);
 	virtual bool OnRequest(SetupRequest &);
-	// TODO
-	//virtual bool OnRequest(LocationRequest &);
-	//virtual bool OnRequest(FacilityRequest &);
 
 private:
 	NumberAnalysisPolicy(const NumberAnalysisPolicy &);
@@ -1072,8 +1183,8 @@ bool NumberAnalysisPolicy::OnRequest(AdmissionRequest & request)
 	if (aliases == NULL)
 		return false;
 
-	for (PINDEX i = 0; i < aliases->GetSize(); i++) {
-		const H225_AliasAddress &alias = (*aliases)[i];
+	for (PINDEX a = 0; a < aliases->GetSize(); a++) {
+		const H225_AliasAddress &alias = (*aliases)[a];
 		if (alias.GetTag() == H225_AliasAddress::e_dialedDigits
 				|| alias.GetTag() == H225_AliasAddress::e_partyNumber) {
 			const PString s = AsString(alias, FALSE);
@@ -1102,8 +1213,8 @@ bool NumberAnalysisPolicy::OnRequest(SetupRequest & request)
 	if (aliases == NULL)
 		return false;
 
-	for (PINDEX i = 0; i < aliases->GetSize(); ++i) {
-		const H225_AliasAddress &alias = (*aliases)[i];
+	for (PINDEX a = 0; a < aliases->GetSize(); ++a) {
+		const H225_AliasAddress &alias = (*aliases)[a];
 		if (alias.GetTag() == H225_AliasAddress::e_dialedDigits
 				|| alias.GetTag() == H225_AliasAddress::e_partyNumber) {
 			const PString s = AsString(alias, FALSE);
@@ -1131,6 +1242,9 @@ class ENUMPolicy : public AliasesPolicy {
 public:
 	ENUMPolicy() { m_name = "ENUM"; }
 protected:
+    virtual bool OnRequest(SetupRequest &) { return false; }
+    virtual bool OnRequest(FacilityRequest &) { return false; }
+
 	virtual bool FindByAliases(RoutingRequest &, H225_ArrayOf_AliasAddress &);
 	virtual bool FindByAliases(LocationRequest &, H225_ArrayOf_AliasAddress &);
 };
@@ -1164,18 +1278,340 @@ bool ENUMPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddre
 		  	}
 		}
 	}
+#else
+	PTRACE(4, "\tENUM policy unavailable as no DNS support.");
 #endif
 
 	return false;
 }
 
-bool ENUMPolicy::FindByAliases(LocationRequest & request, H225_ArrayOf_AliasAddress & aliases)
+bool ENUMPolicy::FindByAliases(LocationRequest & /* request */, H225_ArrayOf_AliasAddress & /* aliases */)
 {
     PTRACE(4, "ROUTING\tPolicy ENUM not supported for LRQ");
 	return false; // ENUMPolicy::FindByAliases((RoutingRequest&)request, aliases);
 }
 
 
+// a policy to route calls via an SQL database
+class SqlPolicy : public Policy {
+public:
+	SqlPolicy();
+	virtual ~SqlPolicy();
+
+protected:
+	virtual bool IsActive() const { return m_active; };
+
+	virtual bool OnRequest(AdmissionRequest &);
+	virtual bool OnRequest(LocationRequest &);
+	virtual bool OnRequest(SetupRequest &);
+
+	virtual void DatabaseLookup(
+		/*in */
+		const PString & source,
+		const PString & calledAlias,
+		const PString & calledIP,
+		const PString & caller,
+		const PString & callid,
+		/* out: */
+		H225_ArrayOf_AliasAddress ** newAliases,	/* DatabaseLookup will allocate, caller must delete */
+		H225_TransportAddress ** newCallSigAdr,		/* DatabaseLookup will allocate, caller must delete */
+		bool * reject);
+
+private:
+	// active ?
+	bool m_active;
+	// connection to the SQL database
+	GkSQLConnection* m_sqlConn;
+	// parametrized query string for the auth condition string retrieval
+	PString m_query;
+	// query timeout
+	long m_timeout;
+};
+
+SqlPolicy::SqlPolicy()
+{
+	m_active = false;
+#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+	m_active = true;
+	static const char *sqlsection = "Routing::Sql";
+	m_name = "SqlPolicy";
+	m_timeout = -1;
+
+	PConfig* cfg = GkConfig();
+
+	const PString driverName = cfg->GetString(sqlsection, "Driver", "");
+	if (driverName.IsEmpty()) {
+		PTRACE(2, m_name << "\tmodule creation failed: "
+			"no SQL driver selected"
+			);
+		m_active = false;
+		return;
+	}
+	
+	m_sqlConn = GkSQLConnection::Create(driverName, m_name);
+	if (m_sqlConn == NULL) {
+		PTRACE(2, m_name << "\tmodule creation failed: "
+			"could not find " << driverName << " database driver"
+			);
+		m_active = false;
+		return;
+	}
+
+	m_query = cfg->GetString(sqlsection, "Query", "");
+	if (m_query.IsEmpty()) {
+		PTRACE(2, m_name << "\tmodule creation failed: "
+			"no query configured"
+			);
+		m_active = false;
+		return;
+	} else
+		PTRACE(4, m_name << "\tQuery: " << m_query);
+		
+	if (!m_sqlConn->Initialize(cfg, sqlsection)) {
+		PTRACE(2, m_name << "\tmodule creation failed: "
+			"could not connect to the database"
+			);
+		return;
+	}
+#else
+	PTRACE(1, m_name << " not available - no database driver compiled into GnuGk");
+#endif // HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+}
+
+SqlPolicy::~SqlPolicy()
+{
+	delete m_sqlConn;
+}
+
+bool SqlPolicy::OnRequest(AdmissionRequest & request)
+{
+	H225_ArrayOf_AliasAddress *aliases = request.GetAliases();
+	H225_AdmissionRequest & arq = request.GetRequest();
+	endptr ep = RegistrationTable::Instance()->FindByEndpointId(arq.m_endpointIdentifier); // should not be null
+	if (ep) {
+		PString source = AsDotString(ep->GetCallSignalAddress());
+		PString calledAlias = "";
+		if (aliases && !aliases->GetSize() == 0)
+			calledAlias = AsString((*aliases)[0], FALSE);
+		PString calledIP = "";	/* not available for ARQs */
+		PString caller = AsString(arq.m_srcInfo, FALSE);
+		PString callid = AsString(arq.m_callIdentifier.m_guid);
+		H225_ArrayOf_AliasAddress * newAliases = NULL;
+		H225_TransportAddress * newCallSigAdr = NULL;
+		bool reject = false;
+
+		DatabaseLookup(	/* in */ source, calledAlias, calledIP, caller, callid,
+						/* out: */ &newAliases, &newCallSigAdr, &reject);
+
+		if (reject) {
+			request.SetFlag(RoutingRequest::e_Reject);
+		}
+
+		if (newAliases) {
+			request.SetFlag(RoutingRequest::e_aliasesChanged);
+			arq.m_destinationInfo = *newAliases;
+			delete newAliases;
+		}
+
+		if (newCallSigAdr) {
+			request.SetFlag(RoutingRequest::e_aliasesChanged);
+			if (!arq.HasOptionalField(H225_AdmissionRequest::e_destCallSignalAddress)) {
+				arq.IncludeOptionalField(H225_AdmissionRequest::e_destCallSignalAddress);
+			}
+			arq.m_destCallSignalAddress = *newCallSigAdr;
+			delete newCallSigAdr;
+		}
+
+		if (m_next == NULL || reject)
+			return true;
+	}
+	return false;
+}
+
+bool SqlPolicy::OnRequest(LocationRequest & request)
+{
+	H225_ArrayOf_AliasAddress *aliases = request.GetAliases();
+	H225_LocationRequest & lrq = request.GetRequest();
+	if (!lrq.HasOptionalField(H225_LocationRequest::e_canMapAlias) || !lrq.m_canMapAlias) {
+			PTRACE(3, "WARNING: Sender can't map destination alias via SqlPolicy");
+	}
+	PString source = AsDotString(lrq.m_replyAddress);
+	PString calledAlias = "";
+	if (aliases && !aliases->GetSize() == 0)
+		calledAlias = AsString((*aliases)[0], FALSE);
+	PString calledIP = "";	/* not available for LRQs */
+	PString caller = "";
+	if (lrq.HasOptionalField(H225_LocationRequest::e_sourceInfo) && (lrq.m_sourceInfo.GetSize() > 0))
+		caller = AsString(lrq.m_sourceInfo[0], FALSE);
+	PString callid = "";	/* not available for LRQs */
+
+	H225_ArrayOf_AliasAddress * newAliases = NULL;
+	H225_TransportAddress * newCallSigAdr = NULL;
+	bool reject = false;
+
+	DatabaseLookup(	/* in */ source, calledAlias, calledIP, caller, callid,
+					/* out: */ &newAliases, &newCallSigAdr, &reject);
+
+	if (reject) {
+		request.SetFlag(RoutingRequest::e_Reject);
+	}
+
+	if (newAliases) {
+		request.SetFlag(RoutingRequest::e_aliasesChanged);
+		lrq.m_destinationInfo = *newAliases;
+		delete newAliases;
+	}
+
+	if (newCallSigAdr) {
+		// 'explicit' policy can't handle LRQs, so we do it directly
+		request.SetFlag(RoutingRequest::e_aliasesChanged);
+		Route route("Sql", *newCallSigAdr);
+		route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(
+			route.m_destAddr
+			);
+		request.AddRoute(route);
+		delete newCallSigAdr;
+		return true;	// stop processing
+	}
+
+	if (m_next == NULL || reject)
+		return true;
+
+	return false;
+}
+
+bool SqlPolicy::OnRequest(SetupRequest & request)
+{
+	H225_Setup_UUIE &setup = request.GetRequest();
+
+	PString source = AsDotString(setup.m_sourceCallSignalAddress);
+	PString calledAlias = "";
+	if (request.GetAliases() && (!request.GetAliases()->GetSize() == 0)) {
+		calledAlias = AsString((*request.GetAliases())[0], FALSE);
+	}
+	PIPSocket::Address localAddr;
+	WORD localPort;
+	request.GetWrapper()->GetLocalAddr(localAddr, localPort);
+	PString calledIP = localAddr;
+	PString caller = AsString(setup.m_sourceAddress, FALSE);
+	PString callid = AsString(setup.m_callIdentifier.m_guid);
+
+	H225_ArrayOf_AliasAddress * newAliases = NULL;
+	H225_TransportAddress * newCallSigAdr = NULL;
+	bool reject = false;
+
+	DatabaseLookup(	/* in */ source, calledAlias, calledIP, caller, callid,
+					/* out: */ &newAliases, &newCallSigAdr, &reject);
+
+	if (reject) {
+		request.SetFlag(RoutingRequest::e_Reject);
+	}
+
+	if (newAliases) {
+		request.SetFlag(RoutingRequest::e_aliasesChanged);
+		if (!setup.HasOptionalField(H225_Setup_UUIE::e_destinationAddress)) {
+			setup.IncludeOptionalField(H225_Setup_UUIE::e_destinationAddress);
+		}
+		setup.m_destinationAddress = *newAliases;
+		delete newAliases;
+	}
+
+	if (newCallSigAdr) {
+		request.SetFlag(RoutingRequest::e_aliasesChanged);
+		if (!setup.HasOptionalField(H225_Setup_UUIE::e_destCallSignalAddress)) {
+			setup.IncludeOptionalField(H225_Setup_UUIE::e_destCallSignalAddress);
+		}
+		setup.m_destCallSignalAddress = *newCallSigAdr;
+		delete newCallSigAdr;
+	}
+
+	if (m_next == NULL || reject)
+		return true;
+
+	return false;
+}
+
+void SqlPolicy::DatabaseLookup(
+		/*in */
+		const PString & source,
+		const PString & calledAlias,
+		const PString & calledIP,
+		const PString & caller,
+		const PString & callid,
+		/* out: */
+		H225_ArrayOf_AliasAddress ** newAliases,	/* DatabaseLookup will allocate, caller must delete */
+		H225_TransportAddress ** newCallSigAdr,	/* DatabaseLookup will allocate, caller must delete */
+		bool * reject)
+{
+#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+	GkSQLResult::ResultRow resultRow;
+	std::map<PString, PString> params;
+	params["s"] = source;
+	params["c"] = calledAlias;
+	params["p"] = calledIP;
+	params["r"] = caller;
+	params["i"] = callid;
+	GkSQLResult* result = m_sqlConn->ExecuteQuery(m_query, params, m_timeout);
+	if (result == NULL) {
+		PTRACE(2, m_name << ": query failed - timeout or fatal error");
+		return;
+	}
+
+	if (!result->IsValid()) {
+		PTRACE(2, m_name << ": query failed (" << result->GetErrorCode()
+			<< ") - " << result->GetErrorMessage()
+			);
+		delete result;
+		return;
+	}
+	
+	if (result->GetNumRows() < 1)
+		PTRACE(3, m_name << ": query returned no rows");
+	else if (result->GetNumFields() < 1)
+		PTRACE(2, m_name << ": bad-formed query - "
+			"no columns found in the result set"
+			);
+	else if (!result->FetchRow(resultRow) || resultRow.empty())
+		PTRACE(2, m_name << ": query failed - could not fetch the result row");
+	else if (result->GetNumFields() == 1) {
+		PString newDestination = resultRow[0].first;
+		PTRACE(5, m_name << "\tQuery result: " << newDestination);
+		*reject = FALSE;
+		if (newDestination.ToUpper() == "REJECT") {
+			*reject = TRUE;
+		} else if (IsIPAddress(newDestination)) {
+			*newCallSigAdr = new H225_TransportAddress();
+			PStringArray adr_parts = newDestination.Tokenise(":", FALSE);
+			PString ip = adr_parts[0];
+			WORD port = (WORD)(adr_parts[1].AsInteger());
+			if (port == 0)
+				port = GK_DEF_ENDPOINT_SIGNAL_PORT;
+			**newCallSigAdr = SocketToH225TransportAddr(ip, port);
+		} else {
+			*newAliases = new H225_ArrayOf_AliasAddress();
+			(*newAliases)->SetSize(1);
+			H323SetAliasAddress(newDestination, (**newAliases)[0]);
+		}
+	} else if (result->GetNumFields() == 2) {
+		PString newDestinationAlias = resultRow[0].first;
+		PString newDestinationIP = resultRow[1].first;
+		PTRACE(5, m_name << "\tQuery result: " << newDestinationAlias << ", " << newDestinationIP);
+		*newAliases = new H225_ArrayOf_AliasAddress();
+		(*newAliases)->SetSize(1);
+		H323SetAliasAddress(newDestinationAlias, (**newAliases)[0]);
+		*newCallSigAdr = new H225_TransportAddress();
+		PStringArray adr_parts = newDestinationIP.Tokenise(":", FALSE);
+		PString ip = adr_parts[0];
+		WORD port = (WORD)(adr_parts[1].AsInteger());
+		if (port == 0)
+			port = GK_DEF_ENDPOINT_SIGNAL_PORT;
+		**newCallSigAdr = SocketToH225TransportAddr(ip, port);
+	}
+	delete result;
+#endif // HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+}
+
+
 namespace { // anonymous namespace
 	SimpleCreator<ExplicitPolicy> ExplicitPolicyCreator("explicit");
 	SimpleCreator<InternalPolicy> InternalPolicyCreator("internal");
@@ -1184,6 +1620,7 @@ namespace { // anonymous namespace
 	SimpleCreator<VirtualQueuePolicy> VirtualQueuePolicyCreator("vqueue");
 	SimpleCreator<NumberAnalysisPolicy> NumberAnalysisPolicyCreator("numberanalysis");
 	SimpleCreator<ENUMPolicy> ENUMPolicyCreator("enum");
+	SimpleCreator<SqlPolicy> SqlPolicyCreator("sql");
 }
 
 
