@@ -17,7 +17,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef ROUTING_H
-#define ROUTING_H "@(#) $Id: Routing.h,v 1.26 2007/03/10 12:16:09 shorne Exp $"
+#define ROUTING_H "@(#) $Id: Routing.h,v 1.30 2008/01/16 12:14:43 willamowius Exp $"
 
 #include <map>
 #include <list>
@@ -78,15 +78,17 @@ struct Route {
 	int m_proxyMode; /// per-route proxy mode flag
 	unsigned m_flags; /// additional route specific flags
 	unsigned char m_rerouteCauses[16]; /// bit flags to trigger rerouting on particular Q931 causes
+	PString m_destNumber;
 };
 
 class RoutingRequest {
 public:
 	enum Flags {
 		e_aliasesChanged = 1,
-		e_fromInternal = 2,
+		e_fromInternal = 2,	// currently not used
 		e_fromParent = 4,
-		e_fromNeighbor = 8
+		e_fromNeighbor = 8,	// currently not used
+		e_Reject = 16
 	};
 	
 	// note this is not a polymorphic class
@@ -294,13 +296,17 @@ public:
 		H225_ArrayOf_AliasAddress* destinationInfo,
 		/// destinationCallSignalAddr (optionally set by this function on successful return)
 		PString* callSigAdr,
+		/// should the call be rejected modified by this function on return)
+		bool & reject,
 		/// an actual virtual queue name (should be present in destinationInfo too)
 		const PString& vqueue,
 		/// a sequence of aliases for the calling endpoint
 		/// (in the "alias:type[=alias:type]..." format)
 		const PString& sourceInfo,
 		/// the callID as string
-		const PString& callID
+		const PString& callID,
+		/// the called IP for unregistered calls
+		const PString& calledip = "unknown"
 		);
 
 	/** Make a routing decision for a pending route request (inserted
@@ -321,7 +327,9 @@ public:
 		/// CRV of the call associated with the route request
 		unsigned crv,
 		/// callID of the call associated with the route request
-		const PString& callID
+		const PString& callID,
+		/// should this call be rejected
+		bool reject = false
 		);
 
 	/** Make a routing decision for a pending route request (inserted
@@ -341,7 +349,9 @@ public:
 		/// CRV of the call associated with the route request
 		unsigned crv,
 		/// callID of the call associated with the route request
-		const PString& callID
+		const PString& callID,
+		/// should this call be rejected
+		bool reject = false
 		);
 
 	/** Reject a pending route request (inserted by SendRequest).
@@ -391,6 +401,8 @@ private:
 		H225_ArrayOf_AliasAddress* m_agent;
 		/// destinationCallSignallAddress for the target agent - target route IF NOT NULL
 		PString* m_callsignaladdr;
+		/// should this call be rejected
+		bool m_reject;
 		/// a synchronization point for signalling that routing decision
 		/// has been made by the external application
 		PSyncPoint m_sync;
