@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////
 //
-// Toolkit base class for the GnuGK
+// Toolkit base class for the GnuGk
 //
 // This work is published under the GNU Public License (GPL)
 // see file COPYING for details.
@@ -10,11 +10,15 @@
 //////////////////////////////////////////////////////////////////
 
 #if defined(_WIN32) && (_MSC_VER <= 1200)
-#pragma warning(disable:4786) // warning about too long debug symbol off
-#pragma warning(disable:4284)
+  #pragma warning(disable:4786) // warning about too long debug symbol off
+  #pragma warning(disable:4284)
+#endif
+#if defined(_WIN32) && (_MSC_VER > 1300)
+  #pragma warning(disable:4244) // warning about possible loss of data
 #endif
 
 #include <ptlib.h>
+#include <ptclib/pdns.h>
 #include <ptclib/cypher.h>
 #include <h323pdu.h>
 #include <map>
@@ -22,12 +26,19 @@
 #include "gktimer.h"
 #include "h323util.h"
 #include "gkconfig.h"
-#if HAS_MYSQL || HAS_PGSQL
+#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
 #include "gksql.h"
 #endif
 #include "clirw.h"
 #include "capctrl.h"
 #include "Toolkit.h"
+#include "gk_const.h"
+
+#if H323_H350
+  const char *H350Section = "GkH350::Settings";
+  #include <ptclib/pldap.h>
+  #include "h350/h350.h"
+#endif
 
 using namespace std;
 
@@ -253,7 +264,10 @@ void Toolkit::RouteTable::InitTable()
 	// Set default IP according to route table
 	PIPSocket::Address defGW;
 	PIPSocket::GetGatewayAddress(defGW);
-	defAddr = GetLocalAddress(defGW);
+	if (defGW.AsString() == "0.0.0.0")
+		PIPSocket::GetNetworkInterface(defAddr);
+	else
+		defAddr = GetLocalAddress(defGW);
 
 #if PTRACING
 	for (RouteEntry *entry = rtable_begin; entry != rtable_end; ++entry)
@@ -439,6 +453,10 @@ int Toolkit::ProxyCriterion::IsInternal(const Address & ip) const
 
 static const char *RewriteSection = "RasSrv::RewriteE164";
 static const char *AliasRewriteSection = "RasSrv::RewriteAlias";
+static const char *AssignedAliasSection = "RasSrv::AssignedAlias";
+#ifdef h323v6
+static const char *AssignedGatekeeperSection = "RasSrv::AssignedGatekeeper";
+#endif
 
 void Toolkit::RewriteData::AddSection(PConfig *config, const PString & section)
 {
@@ -521,6 +539,8 @@ bool Toolkit::RewriteTool::RewritePString(PString & s) const
 		   changed = true;
 		 } else {
 			 // Check if all numeric then is E164 then strip the domain
+			 // this is used with SRV policy which includes the domain when 
+			 // sending the LRQ. 
 			 PINDEX j;
  		     for (j = 0; j < num.GetLength(); ++j)
 			       if (!isdigit(num[j]))
@@ -644,7 +664,6 @@ void Toolkit::GWRewriteTool::PrintData() {
 	}
 
 	for (PINDEX i = 0; i < m_GWRewrite.GetSize(); ++i) {
-
 		// In
 		for (rule_iterator = m_GWRewrite.GetDataAt(i).m_entry_data.first.begin(); rule_iterator != m_GWRewrite.GetDataAt(i).m_entry_data.first.end(); ++rule_iterator) {
 			PTRACE(3, "GK\t" << m_GWRewrite.GetKeyAt(i) << " (in): " << (*rule_iterator).first << " = " << (*rule_iterator).second);
@@ -654,11 +673,8 @@ void Toolkit::GWRewriteTool::PrintData() {
 		for (rule_iterator = m_GWRewrite.GetDataAt(i).m_entry_data.second.begin(); rule_iterator != m_GWRewrite.GetDataAt(i).m_entry_data.second.end(); ++rule_iterator) {
 			PTRACE(3, "GK\t" << m_GWRewrite.GetKeyAt(i) << " (out): " << (*rule_iterator).first << " = " << (*rule_iterator).second);
 		}
-
 	}
-
 	PTRACE(2, "GK\tLoaded " << m_GWRewrite.GetSize() << " GW entries with rewrite info");
-
 }
 
 
@@ -733,7 +749,6 @@ void Toolkit::GWRewriteTool::LoadConfig(PConfig *config) {
 				sorted_out_strings.push_back(rule);
 			}
 
-
 			// Create the entry
 			gw_entry = new GWRewriteEntry();
 			gw_entry->m_entry_data.first = sorted_in_strings;
@@ -749,9 +764,6 @@ void Toolkit::GWRewriteTool::LoadConfig(PConfig *config) {
 	PrintData();
 }
 
-
-
-
 Toolkit::Toolkit() : Singleton<Toolkit>("Toolkit"), 
 	m_Config(NULL), m_ConfigDirty(false),
 	m_acctSessionCounter(0), m_acctSessionBase((long)time(NULL)),
@@ -982,38 +994,116 @@ void Toolkit::ReloadSQLConfig()
 		queryResult = NULL;
 	}
 			
-
+// Rewrite E164
 	query = m_Config->GetString("SQLConfig", "RewriteE164Query", "");
+	if (!query.IsEmpty()) {
+		PTRACE(4, "SQLCONF\tLoading E164 rewrite rules from SQL database");
+		PStringArray params;
+		params += GKName();
+		queryResult = sqlConn->ExecuteQuery(query, &params);
+		if (queryResult == NULL)
+			PTRACE(0, "SQLCONF\tFailed to load E164 rewrite rules from SQL database: "
+				"timeout or fatal error"
+				);
+		else if (!queryResult->IsValid())
+			PTRACE(0, "SQLCONF\tFailed to load E164 rewrite rules from SQL database ("
+				<< queryResult->GetErrorCode() << "): " << queryResult->GetErrorMessage()
+				);
+		else if (queryResult->GetNumFields() < 2)
+			PTRACE(0, "SQLCONF\tFailed to load E164 rewrite rules from SQL database: "
+				"at least 2 columns must be present in the result set"
+				);
+		else {
+			while (queryResult->FetchRow(params))
+				if (params[0].IsEmpty())
+					PTRACE(1, "SQLCONF\tInvalid E164 rewrite rule found in the SQL "
+						"database: '" << params[0] << '=' << params[1] << '\''
+						);
+				else {
+					m_Config->SetString("RasSrv::RewriteE164", params[0], params[1]);
+					PTRACE(6, "SQLCONF\tRewriteE164 rule read: '" << params[0] 
+						<< '=' << params[1] << '\''
+						);
+				}
+			PTRACE(4, "SQLCONF\t" << queryResult->GetNumRows() << " E164 rewrite rules "
+				"loaded from SQL database"
+				);
+		}
+		delete queryResult;
+		queryResult = NULL;
+	}
+
+// Rewrite Alias Query
+	query = m_Config->GetString("SQLConfig", "RewriteAliasQuery", "");
 	if (!query.IsEmpty()) {
 		PTRACE(4, "SQLCONF\tLoading rewrite rules from SQL database");
 		PStringArray params;
 		params += GKName();
 		queryResult = sqlConn->ExecuteQuery(query, &params);
 		if (queryResult == NULL)
-			PTRACE(0, "SQLCONF\tFailed to load rewrite rules from SQL database: "
+			PTRACE(0, "SQLCONF\tFailed to load Alias rewrite rules from SQL database: "
 				"timeout or fatal error"
 				);
 		else if (!queryResult->IsValid())
-			PTRACE(0, "SQLCONF\tFailed to load rewrite rules from SQL database ("
+			PTRACE(0, "SQLCONF\tFailed to load Alias rewrite rules from SQL database ("
 				<< queryResult->GetErrorCode() << "): " << queryResult->GetErrorMessage()
 				);
 		else if (queryResult->GetNumFields() < 2)
-			PTRACE(0, "SQLCONF\tFailed to load rewrite rules from SQL database: "
+			PTRACE(0, "SQLCONF\tFailed to load Alias rewrite rules from SQL database: "
 				"at least 2 columns must be present in the result set"
 				);
 		else {
 			while (queryResult->FetchRow(params))
 				if (params[0].IsEmpty())
-					PTRACE(1, "SQLCONF\tInvalid rewrite rule found in the SQL "
+					PTRACE(1, "SQLCONF\tInvalid Alias rewrite rule found in the SQL "
 						"database: '" << params[0] << '=' << params[1] << '\''
 						);
 				else {
-					m_Config->SetString("RasSrv::RewriteE164", params[0], params[1]);
-					PTRACE(6, "SQLCONF\tRewrite rule read: '" << params[0] 
+					m_Config->SetString("RasSrv::RewriteAlias", params[0], params[1]);
+					PTRACE(6, "SQLCONF\tRewriteAlias rule read: '" << params[0] 
 						<< '=' << params[1] << '\''
 						);
 				}
-			PTRACE(4, "SQLCONF\t" << queryResult->GetNumRows() << " rewrite rules "
+			PTRACE(4, "SQLCONF\t" << queryResult->GetNumRows() << " Alias rewrite rules "
+				"loaded from SQL database"
+				);
+		}
+		delete queryResult;
+		queryResult = NULL;
+	}
+
+// Assigned Alias Query
+	query = m_Config->GetString("SQLConfig", "AssignedAliasQuery", "");
+	if (!query.IsEmpty()) {
+		PTRACE(4, "SQLCONF\tLoading Assigned Alias rules from SQL database");
+		PStringArray params;
+		params += GKName();
+		queryResult = sqlConn->ExecuteQuery(query, &params);
+		if (queryResult == NULL)
+			PTRACE(0, "SQLCONF\tFailed to load Assigned Alias rules from SQL database: "
+				"timeout or fatal error"
+				);
+		else if (!queryResult->IsValid())
+			PTRACE(0, "SQLCONF\tFailed to load Assigned Alias rules from SQL database ("
+				<< queryResult->GetErrorCode() << "): " << queryResult->GetErrorMessage()
+				);
+		else if (queryResult->GetNumFields() < 2)
+			PTRACE(0, "SQLCONF\tFailed to load Assigned Alias rules from SQL database: "
+				"at least 2 columns must be present in the result set"
+				);
+		else {
+			while (queryResult->FetchRow(params))
+				if (params[0].IsEmpty())
+					PTRACE(1, "SQLCONF\tInvalid Assigned Alias rule found in the SQL "
+						"database: '" << params[0] << '=' << params[1] << '\''
+						);
+				else {
+					m_Config->SetString("RasSrv::AssignedAlias", params[0], params[1]);
+					PTRACE(6, "SQLCONF\tAssignedAlias rule read: '" << params[0] 
+						<< '=' << params[1] << '\''
+						);
+				}
+			PTRACE(4, "SQLCONF\t" << queryResult->GetNumRows() << " Assigned Alias rules "
 				"loaded from SQL database"
 				);
 		}
@@ -1163,7 +1253,7 @@ void Toolkit::ReloadSQLConfig()
 	delete sqlConn;
 	sqlConn = NULL;
 	PTRACE(3, "SQLCONF\tSQL config connection closed");
-#endif // HAS_MYSQL || HAS_PGSQL
+#endif // HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
 }
 
 PConfig* Toolkit::ReloadConfig()
@@ -1190,6 +1280,13 @@ PConfig* Toolkit::ReloadConfig()
 	m_ProxyCriterion.LoadConfig(m_Config);
 	m_Rewrite.LoadConfig(m_Config);
 	m_GWRewrite.LoadConfig(m_Config);
+	m_AssignedEPAliases.LoadConfig(m_Config);
+#ifdef h323v6
+	m_AssignedGKs.LoadConfig(m_Config);
+#endif
+#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+	m_qosMonitor.LoadConfig(m_Config);
+#endif
 	PString GKHome(m_Config->GetString("Home", ""));
 	if (m_GKHome.empty() || !GKHome)
 		SetGKHome(GKHome.Tokenise(",:;", false));
@@ -1257,7 +1354,7 @@ void Toolkit::LoadReasonMap(
 
 }
 
-BOOL Toolkit::MatchRegex(const PString &str, const PString &regexStr)
+bool Toolkit::MatchRegex(const PString &str, const PString &regexStr)
 {
 	PINDEX pos=0;
 	PRegularExpression regex(regexStr, PRegularExpression::Extended);
@@ -1272,7 +1369,670 @@ BOOL Toolkit::MatchRegex(const PString &str, const PString &regexStr)
 	return TRUE;
 }
 
+#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+Toolkit::AssignedAliases::AssignedAliases()
+  : m_sqlactive(false), m_sqlConn(NULL)
+{
+}
 
+Toolkit::AssignedAliases::~AssignedAliases()
+{
+}
+
+bool Toolkit::AssignedAliases::LoadSQL(PConfig * cfg)
+{
+	delete m_sqlConn;
+	PString authName = "AssignedAliases::SQL";
+
+   if (cfg->GetSections().GetStringsIndex(authName) == P_MAX_INDEX)
+		return false;
+
+	const PString driverName = cfg->GetString(authName, "Driver", "");
+	if (driverName.IsEmpty()) {
+		PTRACE(0, "AliasSQL\tModule creation failed: "
+			"no SQL driver selected"
+			);
+		PTRACE(0, "AliasSQL\tFATAL: Shutting down");
+		return false;
+	}
+	
+	m_sqlConn = GkSQLConnection::Create(driverName, authName);
+	if (m_sqlConn == NULL) {
+		PTRACE(0, "AliasSQL\tModule creation failed: "
+			"Could not find " << driverName << " database driver"
+			);
+		PTRACE(0, "AliasSQL\tFATAL: Shutting down");
+		return false;
+	}
+		
+	m_query = cfg->GetString(authName, "Query", "");
+	if (m_query.IsEmpty()) {
+		PTRACE(0, "AliasSQL\tModule creation failed: No query configured"
+			);
+		PTRACE(0, "AliasSQL\tFATAL: Shutting down");
+		return false;
+	} else
+		PTRACE(4, "AliasSQL\tQuery: " << m_query);
+		
+	if (!m_sqlConn->Initialize(cfg, authName)) {
+		PTRACE(0, "AliasSQL\tModule creation failed: Could not connect to the database"
+			);
+		return false;
+	}
+
+	return true;
+}
+
+bool Toolkit::AssignedAliases::DatabaseLookup(
+		const PString & alias,
+		PStringArray & newAliases	
+        )
+{
+	if (!m_sqlactive) 
+		return false;
+
+	GkSQLResult::ResultRow resultRow;
+	std::map<PString, PString> params;
+	params["u"] = alias;
+	GkSQLResult* result = m_sqlConn->ExecuteQuery(m_query, params, m_timeout);
+	if (result == NULL) {
+		PTRACE(2, "AliasSQL\tQuery failed - timeout or fatal error");
+		return false;
+	}
+
+	if (!result->IsValid()) {
+		PTRACE(2, "AliasSQL\tQuery failed (" << result->GetErrorCode()
+			<< ") - " << result->GetErrorMessage()
+			);
+		delete result;
+		return false;
+	}
+	
+	bool success = false;
+
+	if (result->GetNumRows() < 1)
+		PTRACE(3, "AliasSQL\tQuery returned no rows");
+	else if (result->GetNumFields() < 1)
+		PTRACE(2, "AliasSQL\tBad-formed query - "
+			"no columns found in the result set"
+			);
+	else {
+		PStringArray retval;
+		while (result->FetchRow(retval)) {
+			if (retval[0].IsEmpty()) {
+				PTRACE(1, "AliasSQL\tQuery Invalid value found.");
+				continue;
+			} 
+		    if (!success) success = true;
+		    PTRACE(5, "AliasSQL\tQuery result: " << retval[0]);
+		    newAliases.AppendString(retval[0]);
+		}
+	}
+	delete result;
+
+   return success;
+}
+#endif   // HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+
+void Toolkit::AssignedAliases::LoadConfig(PConfig * m_config)
+{
+	gkAssignedAliases.clear();
+
+#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+  if (LoadSQL(m_config)) {
+	  m_sqlactive = true;
+  } else 
+#endif
+  {
+   	const PStringToString kv = m_config->GetAllKeyValues(AssignedAliasSection);
+	for (PINDEX i=0; i < kv.GetSize(); i++) {
+		PString data = kv.GetDataAt(i);
+		PStringArray datalines = data.Tokenise(" ,;\t");
+		for (PINDEX j=0; j < datalines.GetSize(); j++) 
+		   gkAssignedAliases.push_back(std::pair<PString, PString>(kv.GetKeyAt(i),datalines[j]));
+	}
+  }
+}
+
+#ifdef H323_H350
+
+bool Toolkit::CreateH350Session(H350_Session * session)
+{
+	PString ldap = GkConfig()->GetString(H350Section, "ServerName", "127.0.0.1");
+	PString port = GkConfig()->GetString(H350Section, "ServerPort", "389");
+	PString server = ldap + ":" + port;
+
+	PString user = GkConfig()->GetString(H350Section, "BindUserDN", "");
+	PString password = Toolkit::Instance()->ReadPassword(H350Section, "BindUserPW");
+    PCaselessString mode = GkConfig()->GetString(H350Section, "BindAuthMode", "simple");
+
+	PLDAPSession::AuthenticationMethod authMethod = PLDAPSession::AuthSimple;
+	if (mode == "sasl")
+		authMethod = PLDAPSession::AuthSASL;
+	else if (mode == "kerberos")
+		authMethod = PLDAPSession::AuthKerberos;
+
+	if (!session->Open(server)) {
+	   PTRACE(4,"H350\tCannot locate H.350 Server");
+	   return false;
+	}
+	if (!user.IsEmpty())
+	    session->Bind(user,password,authMethod);
+
+	return true;
+}
+
+bool Toolkit::AssignedAliases::QueryH350Directory(const PString & alias, PStringArray & aliases)
+{
+// Support Assigned Aliases
+    if (!GkConfig()->GetString(H350Section, "AssignedAliases", "0"))
+		   return false;
+
+// Search the Directory
+	PString search = GkConfig()->GetString(H350Section, "SearchBaseDN", "");
+
+	H225_AliasAddress aliasaddress;
+	H323SetAliasAddress(alias, aliasaddress);
+
+	PString filter;
+	switch (aliasaddress.GetTag()) {
+	  case H225_AliasAddress::e_dialedDigits:
+            filter = "h323IdentitydialedDigits=" + alias;
+	  case H225_AliasAddress::e_h323_ID:
+            filter = "h323Identityh323-ID=" + alias;
+	  case H225_AliasAddress::e_url_ID:
+		    filter = "h323IdentityURL-ID=" + alias;
+	  default:
+		  return false;
+	}
+
+	H350_Session session;
+	if (!Toolkit::Instance()->CreateH350Session(&session)) {
+	   PTRACE(4,"H350\tAssigned Alias: Could not connect to Server.");
+	   return false;
+	}
+
+	H350_Session::LDAP_RecordList rec;
+	int count = session.Search(search,filter,rec);
+	if (count <= 0) {
+	   PTRACE(4,"H350\tAssigned Alias: No Record Found");
+	   session.Close();
+	   return false;
+	}
+
+// Locate the record
+	for (H350_Session::LDAP_RecordList::const_iterator x = rec.begin(); x != rec.end(); ++x) {			
+       H350_Session::LDAP_Record entry = x->second;
+	   PString al;
+	   PINDEX i;
+       if (session.GetAttribute(entry,"h323Identityh323-ID",al)) {
+		   	PStringList als = al.Lines();
+			for (i=0; i< als.GetSize(); i++)
+				aliases.AppendString(als[i]);
+	   }
+       if (session.GetAttribute(entry,"h323IdentitydialedDigits",al)) {
+		   	PStringList als = al.Lines();
+			for (i=0; i< als.GetSize(); i++)
+				aliases.AppendString(als[i]);
+	   }
+       if (session.GetAttribute(entry,"h323IdentityURL-ID",al)) {
+		   	PStringList als = al.Lines();
+			for (i=0; i< als.GetSize(); i++)
+				aliases.AppendString(als[i]);
+	   }
+	   session.Close();
+	   if (aliases.GetSize() > 0) {
+		   PTRACE(2,"H350\tAssigned Alias: Located " << aliases.GetSize() << " Aliases.");
+		   session.Close();
+		   return true;
+	   }
+	}
+
+	PTRACE(4,"H350\tAssigned Alias: No valid Assigned GK found.");
+	session.Close();
+	return false;
+}
+#endif
+
+bool Toolkit::AssignedAliases::QueryAssignedAliases(const PString & alias, PStringArray & aliases)
+{
+#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+	if (DatabaseLookup(alias, aliases))
+		return true;
+#endif
+#ifdef H323_H350
+	if (QueryH350Directory(alias,aliases))
+		return true;
+#endif
+
+	return false;
+}
+
+bool Toolkit::AssignedAliases::GetAliases(const H225_ArrayOf_AliasAddress & alias, H225_ArrayOf_AliasAddress & aliaslist)
+{
+
+	if (alias.GetSize() == 0)
+		    return false;
+
+    PStringArray newaliases;
+	bool found = false;
+	for (PINDEX h=0; h < alias.GetSize(); h++) {
+	  if (QueryAssignedAliases(H323GetAliasAddressString(alias[h]), newaliases))
+		    found = true;
+	}
+
+	if (!found) {
+		for (PINDEX i=0; i < alias.GetSize(); i++) {
+		  PString search = H323GetAliasAddressString(alias[i]);
+		  
+		  for (unsigned j=0; j < gkAssignedAliases.size(); j++) {
+             PTRACE(5,"Alias\tCompare " << gkAssignedAliases[j].first << " to " << search);
+			  if (gkAssignedAliases[j].first == search) {
+				   newaliases.AppendString(gkAssignedAliases[j].second);
+				   if (!found) found = true;
+			  }
+		  }
+		}
+	}
+
+	// Create the Assigned Alias List
+	if (found) {
+	  // add existing items to the end of the list
+		if (aliaslist.GetSize() > 0) {
+			for (PINDEX l=0; l < aliaslist.GetSize(); l++) {
+			  PString a = H323GetAliasAddressString(aliaslist[l]);
+			    bool located = false;
+			    for (PINDEX m=0; m < newaliases.GetSize(); m++) {
+                       if (newaliases[m] == a) located = true;
+				}
+			   if (!located)
+                  newaliases.AppendString(a);
+			}
+		}
+
+		aliaslist.RemoveAll();
+
+		for (PINDEX k=0; k < newaliases.GetSize(); k++) {
+			H225_AliasAddress * aliasaddress = new H225_AliasAddress;
+			H323SetAliasAddress(newaliases[k], *aliasaddress);
+			aliaslist.Append(aliasaddress);
+		}
+	}
+
+	return found;
+}
+
+////////////////////////////////////////////////////////////////////////
+#ifdef h323v6
+#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+Toolkit::AssignedGatekeepers::AssignedGatekeepers()
+  : m_sqlactive(false), m_sqlConn(NULL)
+{
+}
+
+Toolkit::AssignedGatekeepers::~AssignedGatekeepers()
+{
+}
+
+bool Toolkit::AssignedGatekeepers::LoadSQL(PConfig * cfg)
+{
+	delete m_sqlConn;
+	PString authName = "AssignedGatekeepers::SQL";
+
+   if (cfg->GetSections().GetStringsIndex(authName) == P_MAX_INDEX)
+		return false;
+
+	const PString driverName = cfg->GetString(authName, "Driver", "");
+	if (driverName.IsEmpty()) {
+		PTRACE(0, "AssignSQL\tModule creation failed: "
+			"no SQL driver selected"
+			);
+		PTRACE(0, "AssignSQL\tFATAL: Shutting down");
+		return false;
+	}
+	
+	m_sqlConn = GkSQLConnection::Create(driverName, authName);
+	if (m_sqlConn == NULL) {
+		PTRACE(0, "AssignSQL\tModule creation failed: "
+			"Could not find " << driverName << " database driver"
+			);
+		PTRACE(0, "AssignSQL\tFATAL: Shutting down");
+		return false;
+	}
+		
+	m_query = cfg->GetString(authName, "Query", "");
+	if (m_query.IsEmpty()) {
+		PTRACE(0, "AssignSQL\tModule creation failed: No query configured"
+			);
+		PTRACE(0, "AssignSQL\tFATAL: Shutting down");
+		return false;
+	} else
+		PTRACE(4, "AssignSQL\tQuery: " << m_query);
+		
+	if (!m_sqlConn->Initialize(cfg, authName)) {
+		PTRACE(0, "AssignSQL\tModule creation failed: Could not connect to the database"
+			);
+		return false;
+	}
+
+	return true;
+}
+
+bool Toolkit::AssignedGatekeepers::DatabaseLookup(
+		const PString & alias,
+		const PIPSocket::Address & ipaddr,
+		PStringArray & newGks	
+        )
+{
+	if (!m_sqlactive) 
+		return false;
+
+	GkSQLResult::ResultRow resultRow;
+	std::map<PString, PString> params;
+	params["u"] = alias;
+	params["i"] = ipaddr.AsString();
+	GkSQLResult* result = m_sqlConn->ExecuteQuery(m_query, params, m_timeout);
+	if (result == NULL) {
+		PTRACE(2, "AssignSQL\tQuery failed - timeout or fatal error");
+		return false;
+	}
+
+	if (!result->IsValid()) {
+		PTRACE(2, "AssignSQL\tQuery failed (" << result->GetErrorCode()
+			<< ") - " << result->GetErrorMessage()
+			);
+		delete result;
+		return false;
+	}
+	
+	bool success = false;
+
+	if (result->GetNumRows() < 1)
+		PTRACE(3, "AssignSQL\tQuery returned no rows");
+	else if (result->GetNumFields() < 1)
+		PTRACE(2, "AssignSQL\tBad-formed query - "
+			"no columns found in the result set"
+			);
+	else {
+		PStringArray retval;
+		while (result->FetchRow(retval)) {
+			if (retval[0].IsEmpty()) {
+				PTRACE(1, "AssignSQL\tQuery Invalid value found.");
+				continue;
+			} 
+		    if (!success) success = true;
+		    PTRACE(5, "AssignSQL\tQuery result: " << retval[0]);
+		    newGks.AppendString(retval[0]);
+		}
+	}
+	delete result;
+
+   return success;
+}
+#endif   // HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+
+void Toolkit::AssignedGatekeepers::LoadConfig(PConfig * m_config)
+{
+	assignedGKList.clear();
+
+#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+  if (LoadSQL(m_config)) {
+	  m_sqlactive = true;
+  } else 
+#endif
+  {
+	const PStringToString kv = m_config->GetAllKeyValues(AssignedGatekeeperSection);
+	for (PINDEX i=0; i < kv.GetSize(); i++) 
+		   assignedGKList.push_back(std::pair<PString, PString>(kv.GetKeyAt(i),kv.GetDataAt(i)));
+  }
+}
+
+#ifdef H323_H350
+bool Toolkit::AssignedGatekeepers::QueryH350Directory(const PString & alias,const PIPSocket::Address & ip, PStringArray & addresses)
+{
+
+// Support Gatekeeper discovery
+    if (!GkConfig()->GetString(H350Section, "GatekeeperDiscovery", "0"))
+		   return false;
+
+// Search the Directory
+	PString search = GkConfig()->GetString(H350Section, "SearchBaseDN", "");
+
+	H225_AliasAddress aliasaddress;
+	H323SetAliasAddress(alias, aliasaddress);
+
+	PString filter;
+	switch (aliasaddress.GetTag()) {
+	  case H225_AliasAddress::e_dialedDigits:
+            filter = "h323IdentitydialedDigits=" + alias;
+	  case H225_AliasAddress::e_h323_ID:
+            filter = "h323Identityh323-ID=" + alias;
+	  case H225_AliasAddress::e_url_ID:
+		    filter = "h323IdentityURL-ID=" + alias;
+	  default:
+		  return false;
+	}
+
+	H350_Session session;
+	if (!Toolkit::Instance()->CreateH350Session(&session)) {
+	   PTRACE(4,"H350\tAssigned GK: Could not connect to Server.");
+	   return false;
+	}
+
+	H350_Session::LDAP_RecordList rec;
+	int count = session.Search(search,filter,rec);
+	if (count <= 0) {
+	   PTRACE(4,"H350\tAssigned GK: No Record Found");
+	   session.Close();
+	   return false;
+	}
+
+// Locate the record
+	for (H350_Session::LDAP_RecordList::const_iterator x = rec.begin(); x != rec.end(); ++x) {			
+       H350_Session::LDAP_Record entry = x->second;
+	   PString gk;
+	   if (session.GetAttribute(entry,"h323IdentityGKDomain",gk)) {
+           PTRACE(2,"H350\tAssigned GK: GK located " << gk);
+		   addresses = gk.Lines();
+		   session.Close();
+		   return true;
+	   }
+	}
+
+	PTRACE(4,"H350\tAssigned GK: No valid Assigned GK found.");
+	session.Close();
+	return false;
+}
+#endif
+
+bool Toolkit::AssignedGatekeepers::QueryAssignedGK(const PString & alias,const PIPSocket::Address & ip, PStringArray & addresses)
+{
+#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+	if (DatabaseLookup(alias,ip,addresses))
+		return true;
+#endif
+#ifdef H323_H350
+	if (QueryH350Directory(alias,ip,addresses))
+		return true;
+#endif
+
+	return false;
+}
+
+#ifdef hasSRV
+static PString DNStoIP(const PString & dns)
+{
+   H323TransportAddress iface(dns);
+   PIPSocket::Address ip;
+   WORD port = GK_DEF_UNICAST_RAS_PORT;
+   iface.GetIpAndPort(ip, port); 
+   return PString(ip + ":" + port);
+}
+#endif
+
+bool Toolkit::AssignedGatekeepers::GetAssignedGK(const PString & alias,const PIPSocket::Address & ip, H225_ArrayOf_AlternateGK & gklist)
+{
+	bool found = false;
+	PStringArray assignedGK;
+	found = QueryAssignedGK(alias,ip,assignedGK);
+
+	if (!found) {
+		  for (unsigned j=0; j < assignedGKList.size(); j++) {
+		    PString match = assignedGKList[j].first.Trim();
+			if (match.Left(1) != "^") {
+			  // Do prefix match
+			  if (MatchPrefix(alias,assignedGKList[j].first)) {
+				   assignedGK.AppendString(assignedGKList[j].second);
+				   if (!found) found = true;
+			  }
+			} else {
+	          // Do Regex match for IP address
+			  if (MatchRegex(ip.AsString(),match)) {
+				   assignedGK.AppendString(assignedGKList[j].second);
+				   if (!found) found = true;
+		      }
+			}
+		  }
+	}
+
+	if (found) {
+      PStringArray ipaddresses;
+		for (PINDEX k=0; k < assignedGK.GetSize(); k++) {
+           PString number = assignedGK[k];
+
+			if (IsIPAddress(number)) 
+				ipaddresses.AppendString(number);
+#ifdef hasSRV
+			else {
+				PString xnum = assignedGK[k];
+				if (xnum.Left(5) != "h323:") 
+						xnum = "h323:user@" + xnum;
+
+				PStringList str;
+				if (PDNS::LookupSRV(xnum,"_h323rs._udp.",str)) {
+					PTRACE(4, "AssignGK\t" << str.GetSize() << " SRV Records found" );
+					for (PINDEX i = 0; i < str.GetSize(); i++) {
+						PString newhost = str[i].Right(str[i].GetLength()-5);
+						PTRACE(4, "AssignGK\tDNS SRV converted GK address " << number << " to " << newhost );
+						ipaddresses.AppendString(newhost);
+					}
+				} else {
+						ipaddresses.AppendString(DNStoIP(number));
+				}
+			}
+#endif   
+		}
+
+		for (PINDEX k=0; k < ipaddresses.GetSize(); k++) {
+           PString num = ipaddresses[k];
+           WORD port = GK_DEF_UNICAST_RAS_PORT;
+		   PStringArray tokens = num.Tokenise(":", FALSE);
+		   if (tokens.GetSize() == 2) 
+               port = (WORD)tokens[1].AsUnsigned();
+	        
+		   H225_AlternateGK * alt = new H225_AlternateGK;
+		     alt->m_rasAddress = SocketToH225TransportAddr(PIPSocket::Address(tokens[0]),port);
+		     alt->m_needToRegister = true;
+		     alt->m_priority = k;
+		   gklist.Append(alt);
+		}
+	}
+		
+	return found;
+}
+#endif
+
+/////////////////////////////////////////////////////////////////////////////////////////
+
+#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+Toolkit::QoSMonitor::QoSMonitor()
+  : m_sqlactive(false), m_sqlConn(NULL)
+{
+}
+
+Toolkit::QoSMonitor::~QoSMonitor()
+{
+}
+	    
+void Toolkit::QoSMonitor::LoadConfig(PConfig * cfg)
+{
+	delete m_sqlConn;
+	PString authName = "GkQoSMonitor::SQL";
+
+   if (cfg->GetSections().GetStringsIndex(authName) == P_MAX_INDEX)
+		return;
+
+	const PString driverName = cfg->GetString(authName, "Driver", "");
+	if (driverName.IsEmpty()) {
+		PTRACE(0, "QoSSQL\tModule creation failed: "
+			"no SQL driver selected"
+			);
+		PTRACE(0, "QoSSQL\tFATAL: Shutting down");
+		return;
+	}
+	
+	m_sqlConn = GkSQLConnection::Create(driverName, authName);
+	if (m_sqlConn == NULL) {
+		PTRACE(0, "QoSSQL\tModule creation failed: "
+			"Could not find " << driverName << " database driver"
+			);
+		PTRACE(0, "QoSSQL\tFATAL: Shutting down");
+		return;
+	}
+		
+	m_query = cfg->GetString(authName, "Query", "");
+	if (m_query.IsEmpty()) {
+		PTRACE(0, "QoSSQL\tModule creation failed: No query configured"
+			);
+		PTRACE(0, "QoSSQL\tFATAL: Shutting down");
+		return;
+	} else
+		PTRACE(4, "QoSSQL\tQuery: " << m_query);
+		
+	if (!m_sqlConn->Initialize(cfg, authName)) {
+		PTRACE(0, "QoSSQL\tModule creation failed: Could not connect to the database"
+			);
+		return;
+	}
+
+	m_sqlactive = true;
+
+}
+
+bool Toolkit::QoSMonitor::PostRecord(const std::map<PString, PString>& params)
+{
+	if (!m_sqlactive) 
+		return false;
+
+	GkSQLResult* result = m_sqlConn->ExecuteQuery(m_query, params, m_timeout);
+
+	if (result == NULL) {
+		PTRACE(2, "QoSSQL\tFailed to store QoS Data: Timeout or fatal error");
+		return false;
+	}
+	
+	if (result) {
+		if (result->IsValid()) {
+			if (result->GetNumRows() < 1) {
+				PTRACE(4, "QoSSQL\tFailed to store QoS Data: No rows have been updated"
+					);
+				delete result;
+				return false;
+			}
+		} else {
+			PTRACE(2, "QoSSQL\tfailed to store QoS Data: Err(" << result->GetErrorCode() << ") "
+				<< result->GetErrorMessage() );
+			delete result;
+			return false;
+		}
+	}
+    
+	delete result;
+	return true;
+}
+#endif
+
+/////////////////////////////////////////////////////////////////////////////////////////
 
 bool Toolkit::RewriteE164(H225_AliasAddress &alias)
 {
@@ -1510,100 +2270,12 @@ void Toolkit::GetNetworkFromString(
 	}
 }
 
-PString Toolkit::CypherDecode(const PString & key, const PString & crypto, int s)
-{
-	size_t sz = key.GetLength();
-	if (sz > sizeof(PTEACypher::Key))
-		sz = sizeof(PTEACypher::Key);
-	PTEACypher::Key thekey;
-	memset(&thekey, s, sizeof(PTEACypher::Key));
-	memcpy(&thekey, (const char *)key, sz);
-	PTEACypher cypher(thekey);
-
-	return cypher.Decode(crypto);
-}
-
 PString Toolkit::GenerateAcctSessionId()
 {
 	PWaitAndSignal lock( m_acctSessionMutex );
 	return psprintf(PString("%08x%08x"),m_acctSessionBase,++m_acctSessionCounter);
 }
 
-bool Toolkit::AsTimeInterval(
-	/// formatted time interval string
-	const char* inputString,
-	/// variable to store calculated time interval on success
-	PTimeInterval& interval
-	)
-{
-	if (inputString == NULL)
-		return false;
-
-	PTimeInterval result;
-	bool process_next = true;
-	bool valid = false;
-		
-	while (process_next) {
-		char* strend = const_cast<char*>(inputString);
-		long val = strtol(inputString, &strend, 10);
-		
-		// no tokens found?
-		if (strend == inputString)
-			break;
-		
-		// integer range overflow
-		if ((val == LONG_MIN || val == LONG_MAX) && errno == ERANGE) {
-			valid = false;
-			break;
-		}
-		
-		valid = true;
-		inputString = strend;
-		
-		// the last token
-		if (*inputString == 0) {
-			result += PTimeInterval(val);
-			break;
-		}
-
-		// unit specifier		
-		switch (*inputString)
-		{
-		case 's':
-			result += PTimeInterval(0,val);
-			break;
-		case 'm':
-			result += PTimeInterval(0,0,val);
-			break;
-		case 'h':
-			result += PTimeInterval(0,0,0,val);
-			break;
-		case 'd':
-			result += PTimeInterval(0,0,0,0,val);
-			break;
-		case 'w':
-			result += PTimeInterval(0,0,0,0,val*7);
-			break;
-		case 'M':
-			result += PTimeInterval(0,0,0,0,val*30);
-			break;
-		case 'y':
-			result += PTimeInterval(0,0,0,0,val*365);
-			break;
-		default:
-			result += PTimeInterval(val);
-			process_next = false;
-		}
-		
-		if (process_next)
-			inputString++;
-	}
-	
-	if (valid)
-		interval = result;
-	return valid;
-}
-
 PString Toolkit::AsString(
 	const PTime& tm, /// timestamp to convert into a string
 	const PString& formatStr /// format string to use
@@ -1738,3 +2410,13 @@ unsigned Toolkit::MapH225ReasonToQ931Cause(
 	else
 		return m_H225ReasonToQ931Cause[reason];
 }
+
+#ifdef OpenH323Factory
+PStringList Toolkit::GetAuthenticatorList()
+{
+  PString auth = GkConfig()->GetString("Gatekeeper::Main", "Authenticators", "");
+  PStringArray authlist(auth.Tokenise(" ,;\t"));
+
+  return authlist;
+}
+#endif
