@@ -1,25 +1,25 @@
 //////////////////////////////////////////////////////////////////
 //
-// Toolkit base class for the GnuGK
+// Toolkit class for the GnuGk
 //
 // This work is published under the GNU Public License (GPL)
 // see file COPYING for details.
 // We also explicitely grant the right to link this code
 // with the OpenH323 library.
 //
-// History:
-// 	991227  initial version (Torsten Will, mediaWays)
-//
 //////////////////////////////////////////////////////////////////
 
 #ifndef TOOLKIT_H
-#define TOOLKIT_H "@(#) $Id: Toolkit.h,v 1.42 2006/12/13 15:34:22 zvision Exp $"
+#define TOOLKIT_H "@(#) $Id: Toolkit.h,v 1.55 2007/12/19 01:10:53 willamowius Exp $"
 
 #include <vector>
+#include <ptlib/sockets.h>
 #include "singleton.h"
+#include "pwlib_compat.h"
 
 class H225_AliasAddress;
 class H225_ArrayOf_AliasAddress;
+class H225_ArrayOf_AlternateGK;
 class H225_H221NonStandard;
 class H225_Setup_UUIE;
 class SignalingMsg;
@@ -88,8 +88,15 @@ bool operator==(const PIPSocket::Address &addr, const NetworkAddress &net);
 /// @return	True if the given address is contained withing this network
 bool operator<<(const PIPSocket::Address &addr, const NetworkAddress &net);
 
+#ifdef H323_H350
+class H350_Session;
+#endif
+#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+class GkSQLConnection;
+#endif
 class GkTimerManager;
 class CLIRewrite;
+
 class Toolkit : public Singleton<Toolkit>
 {
  public:
@@ -97,12 +104,6 @@ class Toolkit : public Singleton<Toolkit>
 	explicit Toolkit();
 	virtual ~Toolkit();
 
-	/// returns #basic# for
-	virtual const PString GetName() const { return "basic"; }
-
-	// by cwhuang
-	// The idea was got from OpenGatekeeper,
-	// but entirely implemented from scratch. :)
 	class RouteTable {
 		typedef PIPSocket::Address Address;
 		typedef PIPSocket::InterfaceTable InterfaceTable;
@@ -123,9 +124,6 @@ class Toolkit : public Singleton<Toolkit>
 	protected:
 		class RouteEntry : public PIPSocket::RouteEntry {
 		public:
-#ifndef _WIN32
-			PCLASSINFO( RouteEntry, PIPSocket::RouteEntry )
-#endif
 			RouteEntry(const PString &);
 			RouteEntry(const PIPSocket::RouteEntry &, const InterfaceTable &);
 			bool Compare(const Address *) const;
@@ -163,8 +161,7 @@ class Toolkit : public Singleton<Toolkit>
 
 		void LoadConfig(PConfig *);
 
-	protected:
-		bool IsInternal(const Address & ip) const;
+		int IsInternal(const Address & ip) const;
 	
 	private:
 		bool m_enable;
@@ -174,6 +171,9 @@ class Toolkit : public Singleton<Toolkit>
 	bool ProxyRequired(const PIPSocket::Address & ip1, const PIPSocket::Address & ip2) const
 	{ return m_ProxyCriterion.Required(ip1, ip2); }
 
+	bool IsInternal(const PIPSocket::Address & ip1)
+	{ return (m_ProxyCriterion.IsInternal(ip1) > 0);  }
+
 	// Since PStringToString is not thread-safe,
 	// I write this small class to replace that
 	class RewriteData {
@@ -204,6 +204,103 @@ class Toolkit : public Singleton<Toolkit>
 		RewriteData *m_Rewrite;
 	};
 
+	class AssignedAliases {
+	 public:
+#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+	    AssignedAliases();
+		~AssignedAliases();
+
+		bool LoadSQL(PConfig *);
+		bool DatabaseLookup(const PString &,PStringArray &);
+#endif
+	    void LoadConfig(PConfig *);
+		bool QueryAssignedAliases(const PString & alias, PStringArray & aliases);
+#ifdef H323_H350
+		bool QueryH350Directory(const PString & alias, PStringArray & aliases);
+#endif
+		bool GetAliases(const H225_ArrayOf_AliasAddress & alias, H225_ArrayOf_AliasAddress & aliaslist);
+
+	 protected:
+		 std::vector< std::pair<PString, PString> > gkAssignedAliases;
+
+#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+     private:
+	   bool m_sqlactive;
+	   // connection to the SQL database
+	   GkSQLConnection* m_sqlConn;
+	   // parametrized query string for the auth condition string retrieval
+	   PString m_query;
+	   // query timeout
+	   long m_timeout;
+#endif
+	};
+
+	AssignedAliases GetAssignedEPAliases() { return m_AssignedEPAliases; }
+
+#ifdef h323v6
+	class AssignedGatekeepers {
+	 public:
+#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+	    AssignedGatekeepers();
+		~AssignedGatekeepers();
+
+		bool LoadSQL(PConfig *);
+		bool DatabaseLookup(const PString &, const PIPSocket::Address &, PStringArray &);
+#endif
+	    void LoadConfig(PConfig *);
+		bool QueryAssignedGK(const PString & alias, const PIPSocket::Address & ip, PStringArray & addresses);
+#ifdef H323_H350
+		bool QueryH350Directory(const PString & alias,const PIPSocket::Address & ip, PStringArray & addresses);
+#endif
+        bool GetAssignedGK(const PString & alias,const PIPSocket::Address & ip, H225_ArrayOf_AlternateGK & gklist);
+
+	 protected:
+		 std::vector< std::pair<PString, PString> > assignedGKList;
+
+#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+     private:
+	   bool m_sqlactive;
+	   // connection to the SQL database
+	   GkSQLConnection* m_sqlConn;
+	   // parametrized query string for the auth condition string retrieval
+	   PString m_query;
+	   // query timeout
+	   long m_timeout;
+#endif
+
+	};
+
+	AssignedGatekeepers AssignedGKs() { return m_AssignedGKs; }
+#endif
+
+#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+	class QoSMonitor {
+	  public:
+        QoSMonitor();
+		~QoSMonitor();
+	    void LoadConfig(PConfig *);
+		bool PostRecord(const std::map<PString, PString>& params);
+		bool Enabled() { return m_sqlactive; }
+
+     private:
+	   bool m_sqlactive;
+	   // connection to the SQL database
+	   GkSQLConnection* m_sqlConn;
+	   // parametrized query string for the auth condition string retrieval
+	   PString m_query;
+	   // query timeout
+	   long m_timeout;
+	};
+
+	QoSMonitor QoS() { return m_qosMonitor; }
+#endif
+
+
+#ifdef H323_H350
+	// Create H350 connection
+    bool CreateH350Session(H350_Session * session);
+#endif
+
 	/// maybe modifies #alias#. returns true if it did
 	bool RewriteE164(H225_AliasAddress & alias);
 	bool RewriteE164(H225_ArrayOf_AliasAddress & aliases);
@@ -298,57 +395,45 @@ class Toolkit : public Singleton<Toolkit>
 	 * @param regexStr the string which is compiled to a regex and executed with #regex.Execute(str, pos)#
 	 * @return TRUE if the regex matched and FALSE if not or any error case.
 	 */
-	static BOOL MatchRegex(const PString &str, const PString &regexStr);
+	static bool MatchRegex(const PString &str, const PString &regexStr);
 
-	/** returns the #BOOL# that #str# represents.
+	/** returns the #bool# that #str# represents.
 	 * Case insensitive, "t...", "y...", "a...", "1" are #TRUE#, all other values are #FALSE#.
 	 */
 	static bool AsBool(const PString & str);
-	
-	/** Convert a string to a time interval. The string should be formatted like:
-			"1d5h" (1 day and 5 hours) or "20s" (20 seconds) or "1h" (1 hour)
-			or "2000" (2000 milliseconds) or ...
-		General format is a sequence of number and unit specifier pairs.
-		Recognized unit specifiers:
-			y - years
-			M - months
-			w - weeks
-			d - days
-			h - hours
-			m - minutes
-			s - seconds
-			"empty" - milliseconds
-
-		@return
-		true if the input string contained properly formatted time interval,
-		false if no conversion has been made
-	*/
-	static bool AsTimeInterval(
-		/// formatted time interval string
-		const char* inputString,
-		/// variable to store calculated time interval on success
-		PTimeInterval& interval
-		);
 
 	static void GetNetworkFromString(const PString &, PIPSocket::Address &, PIPSocket::Address &);
 
-	static PString CypherDecode(const PString &, const PString &, int);
-
 	/** you may add more extension codes in descendant classes. This codes will not be transferred
 	 * or something it will be the return code of some methods for handling switches easy. */
 	enum {
-		iecUnknown     = -1,  /// internal extension code for an unknown triple(cntry,ext,manuf)
+		iecUnknown     = -1,   /// internal extension code for an unknown triple(cntry,ext,manuf)
 		iecFailoverRAS  = 1,   /// i.e.c. for "This RQ is a failover RQ" and must not be answerd.
-		iecUserbase    = 1000 /// first guaranteed unused 'iec' by GnuGK Toolkit.
+		iecNeighborId = 2,     /// i.e.c. for neighbor id sent inside LRQ/LCF nonStandardData parameter
+		iecNATTraversal = 3,   /// i.e.c. for IP=/NAT= parameters sent inside RRQ/RCF nonStandardData 
+		iecUserbase    = 1000 /// first guaranteed unused 'iec' by GnuGk Toolkit.
 	};
 	/** t35 extension or definitions as field for H225_NonStandardIdentifier */
-	enum {
+	enum T35CountryCodes {
 		t35cOpenOrg = 255,       /// country code for the "Open Source Organisation" Country
+		t35cPoland = 138,
+		t35cUSA = 181
+	};
+	enum T35ManufacturerCodes {
 		t35mOpenOrg = 4242,     /// manufacurers code for the "Open Source Organisation"
+		t35mGnuGk = 2,
+		t35mCisco = 18
+	};
+	enum T35OpenOrgExtensions {
 		t35eFailoverRAS = 255  /// Defined HERE!
 	};
+	enum T35GnuGkExtensions {
+		t35eNeighborId = 1,
+		t35eNATTraversal = 2
+	};
+	
 	/** If the triple #(country,extension,manufacturer)# represents an
-	 * extension known to the GnuGK this method returns its 'internal extension code'
+	 * extension known to the GnuGk this method returns its 'internal extension code'
 	 # #iecXXX' or #iecUnknow# otherwise.
 	 *
 	 * Overwriting methods should use a simlilar scheme and call
@@ -367,11 +452,6 @@ class Toolkit : public Singleton<Toolkit>
 
 	int GetInternalExtensionCode(const H225_H221NonStandard& data) const;
 
-	/** A c-string (#char*#) hash function that considers the
-	 * whole string #name# ending with #\0#.
-	 */
-	inline static unsigned long HashCStr(const unsigned char *name) ;
-
 	/** Generate a call id for accounting purposes, that is unique
 		during subsequent GK start/stop events.
 
@@ -432,11 +512,25 @@ class Toolkit : public Singleton<Toolkit>
 	void SetRerouteCauses(
 		unsigned char *causeMap
 		);
+
+	/** Map H225_ReleaseCompleteReason code to Q.931 cause value.
+	
+	@return
+	The corresponding Q.931 cause value or 0, if there is no direct mapping.
+	*/	
+	unsigned MapH225ReasonToQ931Cause(
+		int reason
+		);
+
+#ifdef OpenH323Factory
+	PStringList GetAuthenticatorList();
+#endif
 		
 protected:
 	void CreateConfig();
 	void ReloadSQLConfig();
 	void LoadCauseMap(PConfig *cfg);
+	void LoadReasonMap(PConfig *cfg);
 	
 	PFilePath m_ConfigFilePath;
 	PFilePath m_extConfigFilePath;
@@ -447,7 +541,13 @@ protected:
 
 	RewriteTool m_Rewrite;
 	GWRewriteTool m_GWRewrite; // GW Based RewriteTool
-
+	AssignedAliases m_AssignedEPAliases;  // Assigned Aliases
+#ifdef h323v6
+	AssignedGatekeepers m_AssignedGKs;
+#endif
+#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+	QoSMonitor m_qosMonitor;
+#endif
 	RouteTable m_RouteTable;
 	VirtualRouteTable m_VirtualRouteTable;
 	ProxyCriterion m_ProxyCriterion;
@@ -475,21 +575,11 @@ private:
 	CLIRewrite *m_cliRewrite;
 	/// bit flag failover triggers for 128 Q931 causes
 	unsigned char m_causeMap[16];
+	/// map H.225 reason to Q.931 cause code
+	vector<unsigned> m_H225ReasonToQ931Cause;
 };
 
 
-inline unsigned long
-Toolkit::HashCStr(const unsigned char *name)
-{
-	register unsigned long h = 0, g;
-	while (*name) {
-		h = (h << 4) + *name++;
-		if ( (g = (h & 0xf0000000)) ) h ^= g >> 24;
-		h &= ~g;
-	}
-	return h;
-}
-
 inline PConfig *GkConfig()
 {
 	return Toolkit::Instance()->Config();
