@@ -3,7 +3,7 @@
  *
  * Module for accoutning per IP/H.323 ID/CLI/prefix inbound call volume
  *
- * $Id: capctrl.cxx,v 1.3 2006/12/13 15:34:22 zvision Exp $
+ * $Id: capctrl.cxx,v 1.8 2007/10/19 20:38:26 zvision Exp $
  *
  * Copyright (c) 2006, Michal Zygmuntowicz
  *
@@ -149,35 +149,26 @@ void CapacityControl::LoadConfig()
 				if (!(ip == "*" || ip == "any"))
 					addr = NetworkAddress(ip);
 
-				ipCallVolumes.resize(ipCallVolumes.size() + 1);
-				ipRule = ipCallVolumes.end() - 1;
-				ipRule->first = addr;
-				ipRule->second.m_sourceAddress = addr;
+				ipCallVolumes.push_back(IpCallVolume(addr, InboundIPCallVolume(addr)));
 				newIpRule = true;
 	
-				rule = &(ipRule->second);
+				rule = &(ipCallVolumes.back().second);
 			} else if (key.Find("h323id:") == 0) {
 				H225_AliasAddress alias;
 				const PString h323id = key.Mid(7).Trim();
 				H323SetAliasAddress(h323id, alias, H225_AliasAddress::e_h323_ID);
 
-				h323IdCallVolumes.resize(h323IdCallVolumes.size() + 1);
-				h323IdRule = h323IdCallVolumes.end() - 1;
-				h323IdRule->first = alias;
-				h323IdRule->second.m_sourceH323Id = alias;
+				h323IdCallVolumes.push_back(H323IdCallVolume(alias, InboundH323IdCallVolume(alias)));
 				newH323IdRule = true;
 				
-				rule = &(h323IdRule->second);
+				rule = &(h323IdCallVolumes.back().second);
 			} else if (key.Find("cli:") == 0) {
-				const PString cli = key.Mid(4).Trim();
+				const std::string cli((const char*)(key.Mid(4).Trim()));
 
-				cliCallVolumes.resize(cliCallVolumes.size() + 1);
-				cliRule = cliCallVolumes.end() - 1;
-				cliRule->first = string((const char*)cli);
-				cliRule->second.m_sourceCLI = string((const char*)cli);
+				cliCallVolumes.push_back(CLICallVolume(cli, InboundCLICallVolume(cli)));
 				newCLIRule = true;
 				
-				rule = &(cliRule->second);
+				rule = &(cliCallVolumes.back().second);
 			} else {
 				PTRACE(1, "CAPCTRL\tUknown CapacityControl rule: " << key << '=' 
 					<< kv.GetDataAt(i)
@@ -191,11 +182,11 @@ void CapacityControl::LoadConfig()
 					<< kv.GetDataAt(i)
 					);
 				if (newIpRule)
-					ipCallVolumes.erase(ipRule);
+					ipCallVolumes.pop_back();
 				else if (newH323IdRule)
-					h323IdCallVolumes.erase(h323IdRule);
+					h323IdCallVolumes.pop_back();
 				else if (newCLIRule)
-					cliCallVolumes.erase(cliRule);
+					cliCallVolumes.pop_back();
 				continue;
 			}
 	
@@ -219,49 +210,43 @@ void CapacityControl::LoadConfig()
 	std::stable_sort(cliCallVolumes.begin(), cliCallVolumes.end(), CLIRule_greater());
 
 	// update route entries that have not changed
-	for (unsigned i = 0; i < ipCallVolumes.size(); ++i) {
-		IpCallVolumes::iterator rule = m_ipCallVolumes.begin();
+	{
+		IpCallVolumes::const_iterator rule = m_ipCallVolumes.begin();
 		while (rule != m_ipCallVolumes.end()) {
-			rule = find(
-				rule, m_ipCallVolumes.end(), ipCallVolumes[i]
+			IpCallVolumes::iterator matchingRule = find(
+				ipCallVolumes.begin(), ipCallVolumes.end(), *rule
 				);
-			if (rule == m_ipCallVolumes.end())
-				break;
-			if (rule->second == ipCallVolumes[i].second) {
-				ipCallVolumes[i].second.m_currentVolume = rule->second.m_currentVolume;
-				ipCallVolumes[i].second.m_calls = rule->second.m_calls;
+			if (matchingRule != ipCallVolumes.end() && matchingRule->second == rule->second) {
+				matchingRule->second.m_currentVolume = rule->second.m_currentVolume;
+				matchingRule->second.m_calls = rule->second.m_calls;
 			}
 			++rule;
 		}
 	}
 
-	for (unsigned i = 0; i < h323IdCallVolumes.size(); ++i) {
-		H323IdCallVolumes::iterator rule = m_h323IdCallVolumes.begin();
+	{
+		H323IdCallVolumes::const_iterator rule = m_h323IdCallVolumes.begin();
 		while (rule != m_h323IdCallVolumes.end()) {
-			rule = find(
-				rule, m_h323IdCallVolumes.end(), h323IdCallVolumes[i]
+			H323IdCallVolumes::iterator matchingRule = find(
+				h323IdCallVolumes.begin(), h323IdCallVolumes.end(), *rule
 				);
-			if (rule == m_h323IdCallVolumes.end())
-				break;
-			if (rule->second == h323IdCallVolumes[i].second) {
-				h323IdCallVolumes[i].second.m_currentVolume = rule->second.m_currentVolume;
-				h323IdCallVolumes[i].second.m_calls = rule->second.m_calls;
+			if (matchingRule != h323IdCallVolumes.end() && matchingRule->second == rule->second) {
+				matchingRule->second.m_currentVolume = rule->second.m_currentVolume;
+				matchingRule->second.m_calls = rule->second.m_calls;
 			}
 			++rule;
 		}
 	}
 
-	for (unsigned i = 0; i < cliCallVolumes.size(); ++i) {
-		CLICallVolumes::iterator rule = m_cliCallVolumes.begin();
+	{
+		CLICallVolumes::const_iterator rule = m_cliCallVolumes.begin();
 		while (rule != m_cliCallVolumes.end()) {
-			rule = find(
-				rule, m_cliCallVolumes.end(), cliCallVolumes[i]
+			CLICallVolumes::iterator matchingRule = find(
+				cliCallVolumes.begin(), cliCallVolumes.end(), *rule
 				);
-			if (rule == m_cliCallVolumes.end())
-				break;
-			if (rule->second == cliCallVolumes[i].second) {
-				cliCallVolumes[i].second.m_currentVolume = rule->second.m_currentVolume;
-				cliCallVolumes[i].second.m_calls = rule->second.m_calls;
+			if (matchingRule != cliCallVolumes.end() && matchingRule->second == rule->second) {
+				matchingRule->second.m_currentVolume = rule->second.m_currentVolume;
+				matchingRule->second.m_calls = rule->second.m_calls;
 			}
 			++rule;
 		}
@@ -315,6 +300,32 @@ void CapacityControl::LoadConfig()
 #endif
 }
 
+PString CapacityControl::PrintRules()
+{
+//	std::stringstream strm;   VS2005 version leaks memory!!
+	PStringStream strm;
+	
+	strm << "Per IP call volume rules:" << endl;
+	for (unsigned i = 0; i < m_ipCallVolumes.size(); ++i) {
+		strm << "  src " << m_ipCallVolumes[i].first.AsString() << ":" << endl;
+		strm << "    " << m_ipCallVolumes[i].second.AsString() << endl;
+	}
+
+	strm << "Per H.323 ID call volume rules:" << endl;
+	for (unsigned i = 0; i < m_h323IdCallVolumes.size(); i++) {
+		strm << "  src " << H323GetAliasAddressString(m_h323IdCallVolumes[i].first) << ":" << endl;
+		strm << "    " << m_h323IdCallVolumes[i].second.AsString() << endl;
+	}
+
+	strm << "Per CLI call volume rules:" << endl;
+	for (unsigned i = 0; i < m_cliCallVolumes.size(); i++) {
+		strm << "  src " << m_cliCallVolumes[i].first << ":" << endl;
+		strm << "    " << m_cliCallVolumes[i].second.AsString() << endl;
+	}
+	
+	return strm;
+}
+
 CapacityControl::IpCallVolumes::iterator CapacityControl::FindByIp(
 	const NetworkAddress &srcIp,
 	const PString &calledStationId
@@ -417,64 +428,83 @@ void CapacityControl::LogCall(
 	bool callStart
 	)
 {
-	IpCallVolumes::iterator bestIpMatch = FindByIp(srcIp, calledStationId);
-	if (bestIpMatch != m_ipCallVolumes.end()) {
-		PTRACE(5, "CAPCTRL\tCall #" << callNumber
-			<< " to " << calledStationId << " matched IP rule " << bestIpMatch->first.AsString()
-			<< "\t" << bestIpMatch->second.AsString()
-			);
-		PWaitAndSignal lock(m_updateMutex);
-		if (callStart) {
+	if (callStart) {
+		IpCallVolumes::iterator bestIpMatch = FindByIp(srcIp, calledStationId);
+		if (bestIpMatch != m_ipCallVolumes.end()) {
+			PTRACE(5, "CAPCTRL\tCall #" << callNumber
+				<< " to " << calledStationId << " matched IP rule " << bestIpMatch->first.AsString()
+				<< "\t" << bestIpMatch->second.AsString()
+				);
+			PWaitAndSignal lock(m_updateMutex);
 			++(bestIpMatch->second.m_currentVolume);
 			bestIpMatch->second.m_calls.push_back(callNumber);
-		} else {
-			if (find(bestIpMatch->second.m_calls.begin(), bestIpMatch->second.m_calls.end(), 
-					callNumber) != bestIpMatch->second.m_calls.end()) {
-				--(bestIpMatch->second.m_currentVolume);
-				bestIpMatch->second.m_calls.remove(callNumber);
-			}
+			return;
 		}
-		return;
-	}
 
-	H323IdCallVolumes::iterator bestH323IdMatch = FindByH323Id(srcAlias, calledStationId);
-	if (bestH323IdMatch != m_h323IdCallVolumes.end()) {
-		PTRACE(5, "CAPCTRL\tCall #" << callNumber
-			<< " to " << calledStationId << " matched H323.ID rule " << H323GetAliasAddressString(bestH323IdMatch->first)
-			<< "\t" << bestH323IdMatch->second.AsString()
-			);
-		PWaitAndSignal lock(m_updateMutex);
-		if (callStart) {
+		H323IdCallVolumes::iterator bestH323IdMatch = FindByH323Id(srcAlias, calledStationId);
+		if (bestH323IdMatch != m_h323IdCallVolumes.end()) {
+			PTRACE(5, "CAPCTRL\tCall #" << callNumber
+				<< " to " << calledStationId << " matched H323.ID rule " << H323GetAliasAddressString(bestH323IdMatch->first)
+				<< "\t" << bestH323IdMatch->second.AsString()
+				);
+			PWaitAndSignal lock(m_updateMutex);
 			++(bestH323IdMatch->second.m_currentVolume);
 			bestH323IdMatch->second.m_calls.push_back(callNumber);
-		} else {
-			if (find(bestH323IdMatch->second.m_calls.begin(), bestH323IdMatch->second.m_calls.end(), 
-					callNumber) != bestH323IdMatch->second.m_calls.end()) {
-				--(bestH323IdMatch->second.m_currentVolume);
-				bestH323IdMatch->second.m_calls.remove(callNumber);
-			}
+			return;
 		}
-		return;
-	}
 
-	CLICallVolumes::iterator bestCliMatch = FindByCli(srcCli, calledStationId);
-	if (bestCliMatch != m_cliCallVolumes.end()) {
-		PTRACE(5, "CAPCTRL\tCall #" << callNumber
-			<< " to " << calledStationId << " matched CLI rule " << bestCliMatch->first
-			<< "\t" << bestCliMatch->second.AsString()
-			);
-		PWaitAndSignal lock(m_updateMutex);
-		if (callStart) {
+		CLICallVolumes::iterator bestCliMatch = FindByCli(srcCli, calledStationId);
+		if (bestCliMatch != m_cliCallVolumes.end()) {
+			PTRACE(5, "CAPCTRL\tCall #" << callNumber
+				<< " to " << calledStationId << " matched CLI rule " << bestCliMatch->first
+				<< "\t" << bestCliMatch->second.AsString()
+				);
+			PWaitAndSignal lock(m_updateMutex);
 			++(bestCliMatch->second.m_currentVolume);
 			bestCliMatch->second.m_calls.push_back(callNumber);
-		} else {
-			if (find(bestCliMatch->second.m_calls.begin(), bestCliMatch->second.m_calls.end(), 
-					callNumber) != bestCliMatch->second.m_calls.end()) {
+			return;
+		}
+	} else { // call stop
+		PWaitAndSignal lock(m_updateMutex);
+		
+		IpCallVolumes::iterator bestIpMatch = m_ipCallVolumes.begin();
+		while (bestIpMatch != m_ipCallVolumes.end()) {
+			std::list<PINDEX>::iterator i = find(bestIpMatch->second.m_calls.begin(),
+				bestIpMatch->second.m_calls.end(), callNumber
+				);
+			if (i != bestIpMatch->second.m_calls.end()) {
+				--(bestIpMatch->second.m_currentVolume);
+				bestIpMatch->second.m_calls.erase(i);
+				return;
+			}
+			++bestIpMatch;
+		}
+
+		H323IdCallVolumes::iterator bestH323IdMatch = m_h323IdCallVolumes.begin();
+		while (bestH323IdMatch != m_h323IdCallVolumes.end()) {
+			std::list<PINDEX>::iterator i = find(bestH323IdMatch->second.m_calls.begin(),
+				bestH323IdMatch->second.m_calls.end(), callNumber
+				);
+			if (i != bestH323IdMatch->second.m_calls.end()) {
+				--(bestH323IdMatch->second.m_currentVolume);
+				bestH323IdMatch->second.m_calls.erase(i);
+				return;
+			}
+			++bestH323IdMatch;
+		}
+
+		CLICallVolumes::iterator bestCliMatch = m_cliCallVolumes.begin();
+		while (bestCliMatch != m_cliCallVolumes.end()) {
+			std::list<PINDEX>::iterator i = find(bestCliMatch->second.m_calls.begin(),
+				bestCliMatch->second.m_calls.end(), callNumber
+				);
+			if (i != bestCliMatch->second.m_calls.end()) {
 				--(bestCliMatch->second.m_currentVolume);
-				bestCliMatch->second.m_calls.remove(callNumber);
+				bestCliMatch->second.m_calls.erase(i);
+				return;
 			}
+			++bestCliMatch;
 		}
-		return;
 	}
 }
 
