@@ -3,7 +3,7 @@
  *
  * Module for CLI/ANI manipulation.
  *
- * $Id: clirw.cxx,v 1.6 2005/04/24 16:39:43 zvision Exp $
+ * $Id: clirw.cxx,v 1.16 2007/11/25 23:23:39 willamowius Exp $
  *
  * Copyright (c) 2005, Michal Zygmuntowicz
  *
@@ -34,17 +34,39 @@ namespace {
 const char * const CLIRewriteSection = "RewriteCLI";
 const char * const ProcessSourceAddress = "ProcessSourceAddress";
 const char * const RemoveH323Id = "RemoveH323Id";
-const char * const ReservedKeys[] = { ProcessSourceAddress, RemoveH323Id, NULL };
+const char * const CLIRPolicy = "CLIRPolicy";
+const char * const ReservedKeys[] = { ProcessSourceAddress, RemoveH323Id, CLIRPolicy, NULL };
 }
 
 CLIRewrite::RewriteRule::RewriteRule()
-	: m_matchType(MatchDialedNumber), m_rewriteType(PrefixToNumber) 
+	: m_matchType(MatchDialedNumber), m_rewriteType(PrefixToNumber),
+	m_screeningType(NoScreening), m_manualCLIR(CLIRPassthrough),
+	m_CLIRPolicy(IgnoreCLIR)
 {
 }
 
 PString CLIRewrite::RewriteRule::AsString() const
 {
 	PString s;
+
+	if (m_manualCLIR == RestrictPresentation)
+		s = "restrict";
+	else if (m_manualCLIR == AllowPresentation)
+		s = "allow";
+
+	if (m_CLIRPolicy != IgnoreCLIR) {
+		if (!s)
+			s += ",";
+		if (m_CLIRPolicy == ForwardCLIR)
+			s += "forward";
+ 		else if (m_CLIRPolicy == ApplyCLIRForTerminals)
+			s += "applyforterminals";
+		else
+			s += "apply";
+	}
+	if (!s)
+		s = "pi=" + s + " ";
+	
 	if (m_matchType == MatchDialedNumber)
 		s += "dialed number: ";
 	else if (m_matchType == MatchDestinationNumber)
@@ -68,6 +90,9 @@ PString CLIRewrite::RewriteRule::AsString() const
 		if (++cli != m_cli.end())
 			s += ", ";
 	}
+	if (m_cli.empty())
+		s += (m_screeningType == HideFromTerminals) ? "hide from terminals only" : "hide";
+
 	return s;
 }
 
@@ -125,7 +150,8 @@ struct DoubleIpRule_greater : public std::binary_function<CLIRewrite::DoubleIpRu
 } /* namespace */
 
 CLIRewrite::CLIRewrite()
-	: m_processSourceAddress(true), m_removeH323Id(false)
+	: m_processSourceAddress(true), m_removeH323Id(false),
+	m_CLIRPolicy(RewriteRule::IgnoreCLIR)
 {
 	PConfig *cfg = GkConfig();
 
@@ -217,13 +243,17 @@ CLIRewrite::CLIRewrite()
 				// no address means wildcard "match any"
 				addr = NetworkAddress();
 				
-				PStringArray dataTokens = data.Tokenise(", ", FALSE);
-				if (dataTokens.GetSize() > 1) {
-					data = dataTokens[1];
-					if (!(dataTokens[0] == "*" || dataTokens[0] == "any"))
-						addr = NetworkAddress(dataTokens[0]);
+				PINDEX sepIndex = data.Find('=');
+				if (sepIndex != P_MAX_INDEX) {
+					PString lhs = data.Left(sepIndex).Trim();
+					sepIndex = lhs.FindOneOf(" \t");
+					if (sepIndex != P_MAX_INDEX) {
+						lhs = lhs.Left(sepIndex).Trim();
+						data = data.Mid(sepIndex + 1);
+						if (!(lhs == "*" || lhs == "any"))
+							addr = NetworkAddress(lhs);
+					}
 				}
-				
 				// check if the address is already on the list
 				siprule = diprule->second.begin();
 				while (siprule != diprule->second.end()) {
@@ -251,6 +281,36 @@ CLIRewrite::CLIRewrite()
 				continue;
 			}
 
+			// process CLIR options
+			int manualCLIR = RewriteRule::CLIRPassthrough;
+			int CLIRPolicy = RewriteRule::IgnoreCLIR;
+			
+			data = data.Trim();
+			if (data.Find("pi=") == 0) {
+				data = data.Mid(3);
+				PINDEX sepIndex = data.FindOneOf(" \t");
+				if (sepIndex != P_MAX_INDEX) {
+					PString lhs = data.Left(sepIndex).Trim();
+					data = data.Mid(sepIndex + 1).Trim();
+					PStringArray tokens = lhs.Tokenise(",;", FALSE);
+					for (PINDEX k = 0; k < tokens.GetSize(); ++k)
+						if (tokens[k] *= "allow")
+							manualCLIR = RewriteRule::AllowPresentation;
+						else if (tokens[k] *= "restrict")
+							manualCLIR = RewriteRule::RestrictPresentation;
+						else if (tokens[k] *= "forward")
+							CLIRPolicy = RewriteRule::ForwardCLIR;
+						else if (tokens[k] *= "apply")
+							CLIRPolicy = RewriteRule::AlwaysApplyCLIR;
+						else if (tokens[k] *= "applyforterminals")
+							CLIRPolicy = RewriteRule::ApplyCLIRForTerminals;
+						else
+							PTRACE(1, "CLIRW\tInvalid CLI rewrite rule syntax: " << key << '=' 
+								<< kv.GetDataAt(k) << ", unreconized pi option '" << tokens[k]
+								<< "'"
+								);
+				}
+			}
 			// process CLI/ANI rewrite rule
 			
 			const PINDEX sepIndex = data.Find('=');
@@ -314,23 +374,6 @@ CLIRewrite::CLIRewrite()
 			if (prefix == "any")
 				prefix.erase();
 
-			// get RHS of the rewrite rule, multiple targets will be selected
-			// in random order
-			PStringArray clis = data.Mid(sepIndex + 1).Tokenise(", ", FALSE);
-			if (clis.GetSize() < 1) {
-				PTRACE(1, "CLIRW\tInvalid CLI rewrite rule syntax - at least 1 "
-					"target number has to be present: " << key << '=' << kv.GetDataAt(i)
-					);
-				if (newsiprule)
-					if (inbound)
-						m_inboundRules.erase(siprule);
-					else
-						diprule->second.erase(siprule);
-				if (newdiprule)
-					m_outboundRules.erase(diprule);
-				continue;
-			}
-
 			// check if the rule already exists			
 			RewriteRules::iterator rule = rules->begin();
 			while (rule != rules->end())
@@ -345,12 +388,29 @@ CLIRewrite::CLIRewrite()
 				rule = rules->end() - 1;
 			}
 	
+			rule->m_screeningType = RewriteRule::NoScreening;
 			rule->m_matchType = matchType;
 			rule->m_rewriteType = rewriteType;
 			rule->m_prefix = prefix;		
-			rule->m_cli.resize(clis.GetSize());
-			for (PINDEX j = 0; j < clis.GetSize(); j++)
-				rule->m_cli[j] = clis[j];
+			rule->m_manualCLIR = manualCLIR;
+			rule->m_CLIRPolicy = CLIRPolicy;
+			
+			// get RHS of the rewrite rule, multiple targets will be selected
+			// in random order
+			PStringArray clis = data.Mid(sepIndex + 1).Tokenise(", ", FALSE);
+			if (clis.GetSize() < 1)
+				rule->m_screeningType = RewriteRule::AlwaysHide;
+			else if (clis[0] *= PString("hide"))
+				rule->m_screeningType = RewriteRule::AlwaysHide;
+			else if (clis[0] *= PString("hidefromterminals"))
+				rule->m_screeningType = RewriteRule::HideFromTerminals;
+			
+			if (rule->m_screeningType == RewriteRule::NoScreening) {
+				rule->m_cli.resize(clis.GetSize());
+				for (PINDEX k = 0; k < clis.GetSize(); k++)
+					rule->m_cli[k] = (string)((const char *)(clis[k]));
+			} else
+				rule->m_cli.clear();
 
 			if (inbound)
 				inboundRules++;
@@ -411,11 +471,27 @@ CLIRewrite::CLIRewrite()
 #endif
 
 	m_processSourceAddress = Toolkit::AsBool(
-		cfg->GetString(CLIRewriteSection, "ProcessSourceAddress", "1")
+		cfg->GetString(CLIRewriteSection, ProcessSourceAddress, "1")
 		);
 	m_removeH323Id = Toolkit::AsBool(
-		cfg->GetString(CLIRewriteSection, "RemoveH323Id", "1")
+		cfg->GetString(CLIRewriteSection, RemoveH323Id, "1")
 		);
+	
+	const PString clirPolicy = cfg->GetString(CLIRewriteSection, CLIRPolicy,
+		PString::Empty()
+		);
+	if (clirPolicy *= "applyforterminals")
+		m_CLIRPolicy = RewriteRule::ApplyCLIRForTerminals;
+	else if (clirPolicy *= "apply")
+		m_CLIRPolicy = RewriteRule::AlwaysApplyCLIR;
+	else if (clirPolicy *= "forward")
+		m_CLIRPolicy = RewriteRule::ForwardCLIR;
+	else if (clirPolicy.IsEmpty())
+		m_CLIRPolicy = RewriteRule::IgnoreCLIR;
+	else
+		PTRACE(1, "CLIRW\tSyntax error in the config - an unrecognized "
+			"CLIRPolicy value: '" << clirPolicy << "'"
+			);
 }
 
 void CLIRewrite::InRewrite(
@@ -517,34 +593,49 @@ void CLIRewrite::Rewrite(
 	PString newcli;
 	RewriteRules::const_iterator rule = ipRule.second.begin();
 	while (rule != ipRule.second.end()) {
-		const RewriteRule &r = *rule++;
-		if (!r.m_prefix.empty()) {
+		if (!rule->m_prefix.empty()) {
 			int matchLen = 0;
 			const char *number = NULL;
-			if (r.m_matchType == RewriteRule::MatchCallerNumber)
+			if (rule->m_matchType == RewriteRule::MatchCallerNumber)
 				number = cli;
-			else if (r.m_matchType == RewriteRule::MatchDialedNumber)
+			else if (rule->m_matchType == RewriteRule::MatchDialedNumber)
 				number = dno;
-			else if (!inbound && r.m_matchType == RewriteRule::MatchDestinationNumber)
+			else if (!inbound && rule->m_matchType == RewriteRule::MatchDestinationNumber)
 				number = cno;
 			if (number != NULL) {
-				matchLen = MatchPrefix(number, r.m_prefix.c_str());
-				if (matchLen > 0 && r.m_rewriteType == RewriteRule::NumberToNumber
+				matchLen = MatchPrefix(number, rule->m_prefix.c_str());
+				if (matchLen > 0 && rule->m_rewriteType == RewriteRule::NumberToNumber
 						&& strlen(number) != (unsigned)matchLen)
 					matchLen = 0;
 			}
-			if (matchLen <= 0)
+			if (matchLen <= 0) {
+				++rule;
 				continue;
+			}
 		}
-		if (!r.m_cli.empty()) {
+		if (rule->m_screeningType != RewriteRule::NoScreening)
+			break;
+		if (!rule->m_cli.empty()) {
 			// get the new ANI/CLI
-			newcli = r.m_cli[rand() % r.m_cli.size()].c_str();
+			newcli = rule->m_cli[rand() % rule->m_cli.size()].c_str();
 			// if this is a number range, choose the new ANI/CLI from the range
 			const PINDEX sepIndex = newcli.Find('-');
 			if (sepIndex != P_MAX_INDEX) {
-				PUInt64 low = newcli.Left(sepIndex).AsUnsigned64();
-				PUInt64 high = newcli.Mid(sepIndex + 1).AsUnsigned64();
+				PString lowStr(newcli.Left(sepIndex).Trim());
+				PString highStr(newcli.Mid(sepIndex + 1).Trim());
+				PUInt64 low = lowStr.AsUnsigned64();
+				PUInt64 high = highStr.AsUnsigned64();
 				PUInt64 diff = (low < high) ? (high - low) : (low - high);
+
+				int numLeadingZeros1 = 0;
+				while (numLeadingZeros1 < lowStr.GetLength()
+						&& lowStr[numLeadingZeros1] == '0')
+					++numLeadingZeros1;
+						
+				int numLeadingZeros2 = 0;
+				while (numLeadingZeros2 < highStr.GetLength()
+						&& highStr[numLeadingZeros2] == '0')
+					++numLeadingZeros2;
 				
 				if (diff >= RAND_MAX)
 					diff = PUInt64(rand());
@@ -553,25 +644,103 @@ void CLIRewrite::Rewrite(
 					
 				diff = (low < high) ? (low + diff) : (high + diff);
 				newcli = PString(diff);
+
+				if (lowStr.GetLength() == highStr.GetLength() && (numLeadingZeros1 > 0 || numLeadingZeros2 > 0)) {
+					while (newcli.GetLength() < highStr.GetLength())
+						newcli = PString("0") + newcli;
+				}
+
 				PTRACE(5, "CLIRW\t" << (inbound ? "Inbound" : "Outbound")
 					<< " CLI range rewrite target is '" << newcli << "' selected by the rule "
-					<< r.AsString()
+					<< rule->AsString()
 					);
 			}
-			if (r.m_rewriteType == RewriteRule::PrefixToPrefix
-					&& r.m_matchType == RewriteRule::MatchCallerNumber)
-				newcli = RewriteString(cli, r.m_prefix.c_str(), newcli);
+			if (rule->m_rewriteType == RewriteRule::PrefixToPrefix
+					&& rule->m_matchType == RewriteRule::MatchCallerNumber)
+				newcli = RewriteString(cli, rule->m_prefix.c_str(), newcli);
 			
 			PTRACE(5, "CLIRW\t" << (inbound ? "Inbound" : "Outbound")
-				<< " CLI rewrite to '" << newcli << "' by the rule " << r.AsString()
+				<< " CLI rewrite to '" << newcli << "' by the rule " << rule->AsString()
 				);
 			break;
 		}
+		++rule;
+	}
+
+	if (rule == ipRule.second.end())
+		return;
+
+	bool isTerminal = false;	
+	if (authData && authData->m_call) {
+		endptr callee = authData->m_call->GetCalledParty();
+		if (callee && callee->GetEndpointType().HasOptionalField(H225_EndpointType::e_terminal))
+			isTerminal = true;
+	}
+
+	if (rule->m_manualCLIR == RewriteRule::RestrictPresentation) {
+		presentation = 1;
+		PTRACE(5, "CLIRW\tCLIR forced to 'restricted' by the " << (inbound ? "inbound" : "outbound")
+			<< " rule " << rule->AsString()
+			);
+	} else if (rule->m_manualCLIR == RewriteRule::AllowPresentation) {
+		presentation = 0;
+		PTRACE(5, "CLIRW\tCLIR forced to 'allowed' by the " << (inbound ? "inbound" : "outbound")
+			<< " rule " << rule->AsString()
+			);
 	}
 	
-	if (newcli.IsEmpty())
+	int clirPolicy = rule->m_CLIRPolicy;
+	if (clirPolicy == RewriteRule::IgnoreCLIR)
+		clirPolicy = m_CLIRPolicy;
+
+	int screeningType = rule->m_screeningType;
+	if (rule->m_CLIRPolicy == RewriteRule::AlwaysApplyCLIR
+			|| (rule->m_CLIRPolicy == RewriteRule::ApplyCLIRForTerminals
+				&& isTerminal)) {
+		if (presentation == 1)
+			screeningType = RewriteRule::AlwaysHide;
+		else if (presentation == (unsigned)-1)
+			if (msg.GetUUIEBody().HasOptionalField(H225_Setup_UUIE::e_presentationIndicator)
+					&& msg.GetUUIEBody().m_presentationIndicator.GetTag() == H225_PresentationIndicator::e_presentationRestricted)
+			screeningType = RewriteRule::AlwaysHide;
+	}
+	
+	if (screeningType == RewriteRule::AlwaysHide) {
+		presentation = 1;
+		screening = 3;
+		newcli = "";
+		plan = Q931::UnknownPlan;
+		type = Q931::UnknownType;
+		if (msg.GetQ931().HasIE((Q931::InformationElementCodes)0x6d)) // calling party subaddress IE
+			msg.GetQ931().RemoveIE((Q931::InformationElementCodes)0x6d);
+		if (msg.GetQ931().HasIE(Q931::DisplayIE))
+			msg.GetQ931().RemoveIE(Q931::DisplayIE);
+		PTRACE(5, "CLIRW\tCLI hidden by the " << (inbound ? "inbound" : "outbound")
+			<< " rule " << rule->AsString()
+			);
+	} else if (screeningType == RewriteRule::HideFromTerminals) {
+		presentation = 1;
+		msg.GetQ931().GetCallingPartyNumber(newcli);
+		if (isTerminal) {
+			isTerminal = true;
+			screening = 3;
+			newcli = "";
+			plan = Q931::UnknownPlan;
+			type = Q931::UnknownType;
+			if (msg.GetQ931().HasIE((Q931::InformationElementCodes)0x6d)) // calling party subaddress IE
+				msg.GetQ931().RemoveIE((Q931::InformationElementCodes)0x6d);
+			if (msg.GetQ931().HasIE(Q931::DisplayIE))
+				msg.GetQ931().RemoveIE(Q931::DisplayIE);
+			PTRACE(5, "CLIRW\tCLI hidden by the " << (inbound ? "inbound" : "outbound")
+				<< " rule " << rule->AsString()
+				);
+		}
+	} else if (newcli.IsEmpty())
 		return;
 
+	if (presentation != (unsigned)-1 && screening == (unsigned)-1)
+		screening = 0;
+
 	msg.GetQ931().SetCallingPartyNumber(newcli, plan, type, presentation, screening);
 	msg.SetChanged();
 	if (m_processSourceAddress && msg.GetUUIEBody().HasOptionalField(H225_Setup_UUIE::e_sourceAddress)) {
@@ -590,7 +759,31 @@ void CLIRewrite::Rewrite(
 					sourceAddress.RemoveAt(aliasIndex);
 			sourceAddress.SetSize(sourceAddress.GetSize() + 1);
 		}
-		H323SetAliasAddress(newcli, sourceAddress[sourceAddress.GetSize() - 1]);
+		
+		if (presentation != (unsigned)-1) {
+			msg.GetUUIEBody().IncludeOptionalField(H225_Setup_UUIE::e_presentationIndicator);
+			msg.GetUUIEBody().m_presentationIndicator.SetTag(presentation);
+			msg.GetUUIEBody().IncludeOptionalField(H225_Setup_UUIE::e_screeningIndicator);
+			msg.GetUUIEBody().m_screeningIndicator.SetValue(screening);
+		}
+		
+		if (screeningType == RewriteRule::AlwaysHide) {
+			msg.GetUUIEBody().IncludeOptionalField(H225_Setup_UUIE::e_presentationIndicator);
+			msg.GetUUIEBody().m_presentationIndicator.SetTag(H225_PresentationIndicator::e_presentationRestricted);
+			msg.GetUUIEBody().IncludeOptionalField(H225_Setup_UUIE::e_screeningIndicator);
+			msg.GetUUIEBody().m_screeningIndicator.SetValue(H225_ScreeningIndicator::e_networkProvided);
+			msg.GetUUIEBody().RemoveOptionalField(H225_Setup_UUIE::e_sourceAddress);
+		} else if (screeningType == RewriteRule::HideFromTerminals) {
+			msg.GetUUIEBody().IncludeOptionalField(H225_Setup_UUIE::e_presentationIndicator);
+			msg.GetUUIEBody().m_presentationIndicator.SetTag(H225_PresentationIndicator::e_presentationRestricted);
+			if (isTerminal && !newcli) {
+				msg.GetUUIEBody().IncludeOptionalField(H225_Setup_UUIE::e_screeningIndicator);
+				msg.GetUUIEBody().m_screeningIndicator.SetValue(H225_ScreeningIndicator::e_networkProvided);
+				H323SetAliasAddress(newcli, sourceAddress[sourceAddress.GetSize() - 1]);
+			} else
+				msg.GetUUIEBody().RemoveOptionalField(H225_Setup_UUIE::e_sourceAddress);
+		} else
+			H323SetAliasAddress(newcli, sourceAddress[sourceAddress.GetSize() - 1]);
 		msg.SetUUIEChanged();
 	}
 }
