@@ -12,6 +12,18 @@
  * with the OpenH323 library.
  *
  * $Log: gkacct.cxx,v $
+ * Revision 1.34  2008/01/18 12:17:54  willamowius
+ * add accounting variable %{cause-translated}
+ *
+ * Revision 1.33  2007/09/26 14:57:54  shorne
+ * Added CallLinkage SQL variable
+ *
+ * Revision 1.32  2007/09/11 01:02:29  willamowius
+ * clean up includes
+ *
+ * Revision 1.31  2007/07/14 09:47:42  willamowius
+ * new accounting variable %{bandwidth}
+ *
  * Revision 1.30  2006/04/14 13:56:19  willamowius
  * call failover code merged
  *
@@ -129,7 +141,6 @@
 #include "h323util.h"
 #include "stl_supp.h"
 #include "Toolkit.h"
-#include "RasTbl.h"
 #include "gktimer.h"
 #include "gkacct.h"
 
@@ -239,6 +250,7 @@ void GkAcctLogger::SetupAcctParams(
 	params["u"] = GetUsername(call);
 	params["d"] = call->GetDuration();
 	params["c"] = call->GetDisconnectCause();
+	params["cause-translated"] = call->GetDisconnectCauseTranslated();
 	params["s"] = call->GetAcctSessionId();
 	params["p"] = call->GetPostDialDelay();
 	params["r"] = call->GetReleaseSource();
@@ -249,6 +261,7 @@ void GkAcctLogger::SetupAcctParams(
 		params["gkip"] = interfaces.front().AsString();
 	params["CallId"] = ::AsString(call->GetCallIdentifier().m_guid);
 	params["ConfId"] = ::AsString(call->GetConferenceIdentifier());
+	params["CallLink"] = call->GetCallLinkage();  
 	
 	t = call->GetSetupTime();
 	if (t)
@@ -298,6 +311,7 @@ void GkAcctLogger::SetupAcctParams(
 	if (call->GetMediaOriginatingIp(addr))
 		params["media-oip"] = addr.AsString();
 	params["codec"] = call->GetCodec();
+	params["bandwidth"] = call->GetBandwidth();
 }
 
 PString GkAcctLogger::ReplaceAcctParams(
