@@ -11,6 +11,15 @@
  * with the OpenH323 library.
  *
  * $Log: gksql_firebird.cxx,v $
+ * Revision 1.8  2007/09/10 18:13:48  willamowius
+ * clean up sql driver interface and remove unused methods from all drivers
+ *
+ * Revision 1.7  2007/09/10 11:17:21  willamowius
+ * fix comment
+ *
+ * Revision 1.6  2007/08/15 09:43:14  zvision
+ * Compilation error fixed
+ *
  * Revision 1.5  2007/03/13 18:39:43  willamowius
  * compile fix for PWLib 1.11.3CVS
  *
@@ -96,7 +105,7 @@ public:
 	GkIBSQLResult(
 		/// Firebird specific error code
 		unsigned int errorCode,
-		/// PostgreSQL specific error message text
+		/// Firebird specific error message text
 		const char* errorMsg,
 		/// transaction handle
 		isc_tr_handle tr = NULL,
@@ -118,12 +127,6 @@ public:
 	*/	
 	virtual long GetErrorCode();
 	
-	/** @return
-	    True if rows can be fetched in random access order, false if
-	    rows have to be fethed sequentially and can be retrieved only once.
-	*/
-	virtual bool HasRandomAccess();
-
 	/** Fetch a single row from the result set. After each row is fetched,
 	    cursor position is moved to a next row.
 		
@@ -138,47 +141,7 @@ public:
 		/// array to be filled with string representations of the row fields
 		ResultRow& result
 		);
-
-	/** @return
-	    True if the column at the index #fieldOffset# is NULL in the row 
-	    fetched most recently.
-	*/
-	virtual bool IsNullField(
-		/// index of the column to check
-		long fieldOffset
-		);
 			
-	/** Fetch a single row from the result set. This function requires
-		that the backend supports random row access.
-		
-	    @return
-	    True if the row has been fetched, false if a row at the given offset
-		does not exists or SQL backend does not support random row access.
-	*/
-	virtual bool FetchRow(
-		/// array to be filled with string representations of the row fields
-		PStringArray& result,
-		/// index (0 based) of the row to fetch
-		long rowOffset
-		);
-	virtual bool FetchRow(
-		/// array to be filled with string representations of the row fields
-		ResultRow& result,
-		/// index (0 based) of the row to fetch
-		long rowOffset
-		);
-		
-	/** @return
-	    True if the column at the index #fieldOffset# is NULL in the row 
-	    at the specified index.
-	*/
-	virtual bool IsNullField(
-		/// index of the column to check
-		long fieldOffset,
-		/// index (0 based) of the row to check
-		long rowOffset
-		);
-
 private:
 	GkIBSQLResult();
 	GkIBSQLResult(const GkIBSQLResult&);
@@ -303,8 +266,6 @@ GkIBSQLResult::GkIBSQLResult(
 		}
 	} else
 		m_queryError = true;
-		
-	m_selectType = (m_numFields != 0);
 }
 
 GkIBSQLResult::GkIBSQLResult(
@@ -351,11 +312,6 @@ GkIBSQLResult::~GkIBSQLResult()
 			isc_commit_transaction(status, &m_tr);
 }
 
-bool GkIBSQLResult::HasRandomAccess()
-{
-	return false;
-}
-
 PString GkIBSQLResult::GetErrorMessage()
 {
 	return m_errorMessage;
@@ -461,46 +417,6 @@ bool GkIBSQLResult::FetchRow(
 	return true;
 }
 
-bool GkIBSQLResult::IsNullField(
-	/// index of the column to check
-	long fieldOffset
-	)
-{
-	return m_sqlResult == NULL || m_sqlRow < 0 || m_sqlRow >= m_numRows
-		|| fieldOffset < 0 || fieldOffset >= m_numFields
-		|| ((m_sqlResult->sqlvar[fieldOffset].sqltype & 1) && *(m_sqlResult->sqlvar[fieldOffset].sqlind) == -1);
-}
-
-bool GkIBSQLResult::FetchRow(
-	/// array to be filled with string representations of the row fields
-	PStringArray& result,
-	/// index (0 based) of the row to fetch
-	long rowOffset
-	)
-{
-	return false;
-}
-
-bool GkIBSQLResult::FetchRow(
-	/// array to be filled with string representations of the row fields
-	ResultRow& result,
-	/// index (0 based) of the row to fetch
-	long rowOffset
-	)
-{
-	return false;
-}
-
-bool GkIBSQLResult::IsNullField(
-	/// index of the column to check
-	long fieldOffset,
-	/// index (0 based) of the row to check
-	long rowOffset
-	)
-{
-	return false;
-}
-
 
 GkIBSQLConnection::GkIBSQLConnection(
 	/// name to use in the log
@@ -547,7 +463,7 @@ GkSQLConnection::SQLConnPtr GkIBSQLConnection::CreateNewConnection(
 	
 	ISC_STATUS status[20];
 	isc_db_handle conn = NULL;
-	std::string dbname = m_database;
+	std::string dbname((const char*)m_database);
 	
 	if (!m_host) {
 		dbname.insert(0, ":");
