@@ -12,10 +12,13 @@
  *
  */
 
+#include "pwlib_compat.h"
+
 #ifdef P_ODBC
 
 #include <ptlib.h>
-#include <ptlib/podbc.h>
+#include <ptclib/podbc.h>
+#include "gksql.h"
 
 
 /** Class that encapsulates SQL query result for odbc backend.
@@ -57,12 +60,6 @@ public:
 	*/	
 	virtual long GetErrorCode();
 	
-	/** @return
-	    True if rows can be fetched in random access order, false if
-	    rows have to be fethed sequentially and can be retrieved only once.
-	*/
-	virtual bool HasRandomAccess();
-
 	/** Fetch a single row from the result set. After each row is fetched,
 	    cursor position is moved to a next row.
 		
@@ -78,46 +75,6 @@ public:
 		ResultRow& result
 		);
 
-	/** @return
-	    True if the column at the index #fieldOffset# is NULL in the row 
-	    fetched most recently.
-	*/
-	virtual bool IsNullField(
-		/// index of the column to check
-		long fieldOffset
-		);
-			
-	/** Fetch a single row from the result set. This function requires
-		that the backend supports random row access.
-		
-	    @return
-	    True if the row has been fetched, false if a row at the given offset
-		does not exists or SQL backend does not support random row access.
-	*/
-	virtual bool FetchRow(
-		/// array to be filled with string representations of the row fields
-		PStringArray& result,
-		/// index (0 based) of the row to fetch
-		long rowOffset
-		);
-	virtual bool FetchRow(
-		/// array to be filled with string representations of the row fields
-		ResultRow& result,
-		/// index (0 based) of the row to fetch
-		long rowOffset
-		);
-		
-	/** @return
-	    True if the column at the index #fieldOffset# is NULL in the row 
-	    at the specified index.
-	*/
-	virtual bool IsNullField(
-		/// index of the column to check
-		long fieldOffset,
-		/// index (0 based) of the row to check
-		long rowOffset
-		);
-
 private:
 	GkodbcResult();
 	GkodbcResult(const GkodbcResult&);
@@ -212,6 +169,7 @@ protected:
 private:
 	GkodbcConnection(const GkodbcConnection&);
 	GkodbcConnection& operator=(const GkodbcConnection&);
+
 };
 
 
@@ -227,8 +185,6 @@ GkodbcResult::GkodbcResult(
 		m_numFields = PQnfields(m_sqlResult);
 	} else
 		m_queryError = true;
-		
-	m_selectType = true;
 }
 
 GkodbcResult::GkodbcResult(
@@ -239,7 +195,6 @@ GkodbcResult::GkodbcResult(
 	m_errorCode(0)
 {
 	m_numRows = numRowsAffected;
-	m_selectType = false;
 }
 	
 GkodbcResult::GkodbcResult(
@@ -257,11 +212,6 @@ GkodbcResult::~GkodbcResult()
 {
 }
 
-bool GkodbcResult::HasRandomAccess()
-{
-	return true;
-}
-
 PString GkodbcResult::GetErrorMessage()
 {
 	return m_errorMessage;
@@ -288,44 +238,6 @@ bool GkodbcResult::FetchRow(
 	return false;
 }
 
-bool GkodbcResult::IsNullField(
-	/// index of the column to check
-	long fieldOffset
-	)
-{
-	return false;
-}
-
-bool GkodbcResult::FetchRow(
-	/// array to be filled with string representations of the row fields
-	PStringArray& result,
-	/// index (0 based) of the row to fetch
-	long rowOffset
-	)
-{
-	return false;
-}
-
-bool GkodbcResult::FetchRow(
-	/// array to be filled with string representations of the row fields
-	ResultRow& result,
-	/// index (0 based) of the row to fetch
-	long rowOffset
-	)
-{
-	return false;
-}
-
-bool GkodbcResult::IsNullField(
-	/// index of the column to check
-	long fieldOffset,
-	/// index (0 based) of the row to check
-	long rowOffset
-	)
-{
-	return false;
-}
-
 
 GkodbcConnection::GkodbcConnection(
 	/// name to use in the log
@@ -377,7 +289,8 @@ PString GkodbcConnection::EscapeString(
 }
 
 namespace {
-	GkSQLCreator<GkodbcConnection> PgSQLCreator("odbc");
+	GkSQLCreator<GkodbcConnection> OdbcSQLCreator("odbc");
 }
 
-#endif /* HAS_PGSQL */
+#endif /* P_ODBC */
+
