@@ -12,6 +12,21 @@
  * with the OpenH323 library.
  *
  * $Log: radauth.cxx,v $
+ * Revision 1.42  2008/01/30 19:00:31  willamowius
+ * fix bug introduced when renaming variables
+ *
+ * Revision 1.41  2007/11/25 23:23:40  willamowius
+ * cleanup: rename variables that shadow others with the same name
+ *
+ * Revision 1.40  2007/11/04 03:26:21  shorne
+ * Fix problem with FramedIP with NATed endpoints.
+ *
+ * Revision 1.39  2007/09/11 01:02:29  willamowius
+ * clean up includes
+ *
+ * Revision 1.38  2007/07/10 19:33:27  willamowius
+ * allow Radius server to send multiple destinations in "h323-redirect-number" attribute for call failover (patch by Lucas Martinez)
+ *
  * Revision 1.37  2006/12/06 16:34:04  zvision
  * Handle RADIUS Class attribute correctly
  *
@@ -216,7 +231,6 @@
 #include "Routing.h"
 #include "sigmsg.h"
 #include "radproto.h"
-#include "gkauth.h"
 #include "radauth.h"
 
 using std::vector;
@@ -314,6 +328,7 @@ int RadAuthBase::Check(
 
 	// append Framed-IP-Address					
 	PIPSocket::Address addr;
+	const PIPSocket::Address & rx_addr = rrqPdu->m_peerAddr;
 	bool ipFound = false;
 	if (rrq.m_callSignalAddress.GetSize() > 0) {
 		if (GetIPFromTransportAddr(rrq.m_callSignalAddress[0], addr)
@@ -332,7 +347,7 @@ int RadAuthBase::Check(
 		delete pdu;
 		return e_fail;
 	} else
-		pdu->AppendAttr(RadiusAttr::FramedIpAddress, addr);
+		pdu->AppendAttr(RadiusAttr::FramedIpAddress, (rx_addr != addr)? rx_addr : addr);
 				
 	if (m_appendCiscoAttributes && m_includeTerminalAliases
 			&& rrq.HasOptionalField(H225_RegistrationRequest::e_terminalAlias)) {
@@ -574,7 +589,8 @@ int RadAuthBase::Check(
 		arq.m_answerCall ? RadiusAttr::ST_CallCheck : RadiusAttr::ST_Login
 		);
 				
-	// append Frame-IP-Address					
+	// append Frame-IP-Address	
+    const PIPSocket::Address & rx_addr = arqPdu->m_peerAddr;
 	bool ipFound = false;
 	if (arq.m_answerCall) {
 		if (calledEP 
@@ -603,7 +619,7 @@ int RadAuthBase::Check(
 		delete pdu;
 		return e_fail;
 	} else
-		pdu->AppendAttr(RadiusAttr::FramedIpAddress, addr);
+		pdu->AppendAttr(RadiusAttr::FramedIpAddress, (rx_addr != addr)? rx_addr : addr);
 					
 	// fill Calling-Station-Id and Called-Station-Id fields
 	PString stationId = GetCallingStationId(arqPdu, authData);
@@ -723,9 +739,9 @@ int RadAuthBase::Check(
 	}
 	// check for Session-Timeout attribute (alternate call duration limit)	
 	if (result) {
-		const RadiusAttr* const attr = response->FindAttr(RadiusAttr::SessionTimeout);
-		if (attr != NULL) {
-			const long sessionTimeout = attr->AsInteger();
+		const RadiusAttr* const tattr = response->FindAttr(RadiusAttr::SessionTimeout);
+		if (tattr != NULL) {
+			const long sessionTimeout = tattr->AsInteger();
 			if (authData.m_callDurationLimit < 0 
 				|| authData.m_callDurationLimit > sessionTimeout) {
 				authData.m_callDurationLimit = sessionTimeout;
@@ -790,7 +806,7 @@ int RadAuthBase::Check(
 			}
 		}
 	}
-
+	PStringArray numbersToDial;
 	// check for h323-redirect-number
 	if (result) {
 		attr = response->FindVsaAttr(RadiusAttr::CiscoVendorId, 
@@ -799,7 +815,8 @@ int RadAuthBase::Check(
 		if (attr != NULL) {
 			value = attr->AsCiscoString();
 			if (!value) {
-				authData.SetRouteToAlias(value);
+				numbersToDial = value.Tokenise("; \t", FALSE);
+				authData.SetRouteToAlias(numbersToDial[0]);
 				PTRACE(5, "RADAUTH\t" << GetName() << " ARQ check redirect "
 					"to the number " << value
 					);
@@ -817,15 +834,16 @@ int RadAuthBase::Check(
 			if (!value) {
 				PStringArray tokens(value.Tokenise("; \t", FALSE));
 				for (PINDEX i = 0; i < tokens.GetSize(); ++i) {
-					PIPSocket::Address addr;
-					WORD port = 0;
+					PIPSocket::Address raddr;
+					WORD rport = 0;
 					
-					if (GetTransportAddress(tokens[i], GK_DEF_ENDPOINT_SIGNAL_PORT, addr, port)
-							&& addr.IsValid() && port != 0) {
-						Route route("RADIUS", addr, port);
+					if (GetTransportAddress(tokens[i], GK_DEF_ENDPOINT_SIGNAL_PORT, raddr, rport)
+							&& raddr.IsValid() && rport != 0) {
+						Route route("RADIUS", raddr, rport);
 						route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(
-							SocketToH225TransportAddr(addr, port)
+							SocketToH225TransportAddr(raddr, rport)
 							);
+						route.m_destNumber = (i < numbersToDial.GetSize()) ? numbersToDial[i] : numbersToDial[numbersToDial.GetSize() - 1];
 						authData.m_destinationRoutes.push_back(route);
 						PTRACE(5, "RADAUTH\t" << GetName() << " ARQ check redirect "
 							"to the address " << route.AsString()
@@ -1030,9 +1048,9 @@ int RadAuthBase::Check(
 	}
 	// check for Session-Timeout attribute (alternate call duration limit)	
 	if (result) {
-		const RadiusAttr* const attr = response->FindAttr(RadiusAttr::SessionTimeout);
-		if (attr != NULL) {
-			const long sessionTimeout = attr->AsInteger();
+		const RadiusAttr* const tattr = response->FindAttr(RadiusAttr::SessionTimeout);
+		if (tattr != NULL) {
+			const long sessionTimeout = tattr->AsInteger();
 			if (authData.m_callDurationLimit < 0 
 				|| authData.m_callDurationLimit > sessionTimeout) {
 				authData.m_callDurationLimit = sessionTimeout;
@@ -1045,6 +1063,7 @@ int RadAuthBase::Check(
 		}
 	}
 
+	PStringArray numbersToDial;
 	// check for h323-redirect-number
 	if (result) {
 		attr = response->FindVsaAttr(RadiusAttr::CiscoVendorId, 
@@ -1053,7 +1072,8 @@ int RadAuthBase::Check(
 		if (attr != NULL) {
 			value = attr->AsCiscoString();
 			if (!value) {
-				authData.SetRouteToAlias(value);
+				numbersToDial = value.Tokenise("; \t", FALSE);
+				authData.SetRouteToAlias(numbersToDial[0]);
 				PTRACE(5, "RADAUTH\t" << GetName() << " ARQ check redirect "
 					"to the number " << value
 					);
@@ -1071,15 +1091,16 @@ int RadAuthBase::Check(
 			if (!value) {
 				PStringArray tokens(value.Tokenise("; \t", FALSE));
 				for (PINDEX i = 0; i < tokens.GetSize(); ++i) {
-					PIPSocket::Address addr;
-					WORD port = 0;
+					PIPSocket::Address raddr;
+					WORD rport = 0;
 					
-					if (GetTransportAddress(tokens[i], GK_DEF_ENDPOINT_SIGNAL_PORT, addr, port)
-							&& addr.IsValid() && port != 0) {
-						Route route("RADIUS", addr, port);
+					if (GetTransportAddress(tokens[i], GK_DEF_ENDPOINT_SIGNAL_PORT, raddr, rport)
+							&& raddr.IsValid() && rport != 0) {
+						Route route("RADIUS", raddr, rport);
 						route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(
-							SocketToH225TransportAddr(addr, port)
+							SocketToH225TransportAddr(raddr, rport)
 							);
+						route.m_destNumber = (i < numbersToDial.GetSize()) ? numbersToDial[i] : numbersToDial[numbersToDial.GetSize() - 1];
 						authData.m_destinationRoutes.push_back(route);
 						PTRACE(5, "RADAUTH\t" << GetName() << " Setup check redirect "
 							"to the address " << route.AsString()
@@ -1089,7 +1110,7 @@ int RadAuthBase::Check(
 			}
 		}
 	}
-			
+
 	if (result)
 		result = OnReceivedPDU(*response, setup, authData);
 	else
