@@ -11,6 +11,21 @@
  * with the OpenH323 library.
  *
  * $Log: sqlacct.cxx,v $
+ * Revision 1.14  2007/09/28 22:20:23  willamowius
+ * cleanup includes
+ *
+ * Revision 1.13  2006/04/14 13:56:19  willamowius
+ * call failover code merged
+ *
+ * Revision 1.1.1.1  2005/11/21 20:20:00  willamowius
+ *
+ *
+ * Revision 1.4  2005/11/15 19:52:56  jan
+ * Michal v1 (works, but on in routed, not proxy mode)
+ *
+ * Revision 1.12  2005/05/19 16:41:17  zvision
+ * Solaris need explicit std::map
+ *
  * Revision 1.11  2005/04/24 16:39:45  zvision
  * MSVC6.0 compatibility fixed
  *
@@ -68,20 +83,13 @@
 #pragma warning(disable:4284)
 #endif
 
+#include <vector>
 #include <ptlib.h>
-#include <ptlib/sockets.h>
-#include <h225.h>
-#include "gk_const.h"
-#include "h323util.h"
-#include "Toolkit.h"
-#include "RasTbl.h"
 #include "RasSrv.h"
 #include "gksql.h"
-#include "gkacct.h"
 #include "sqlacct.h"
 
 using std::vector;
-using std::map;
 
 
 SQLAcct::SQLAcct(
@@ -231,7 +239,7 @@ GkAcctLogger::Status SQLAcct::Log(
 		return Fail;
 	}
 
-	map<PString, PString> params;
+	std::map<PString, PString> params;
 	SetupAcctParams(params, call, m_timestampFormat);
 	GkSQLResult* result = m_sqlConn->ExecuteQuery(query, params);
 	if (result == NULL)
