@@ -3,7 +3,7 @@
  *
  * SQL authentication/authorization modules for GNU Gatekeeper
  *
- * $Id: sqlauth.cxx,v 1.10 2006/04/14 13:56:19 willamowius Exp $
+ * $Id: sqlauth.cxx,v 1.12 2007/11/25 23:23:40 willamowius Exp $
  *
  * Copyright (c) 2004, Michal Zygmuntowicz
  *
@@ -622,6 +622,7 @@ int SQLAuth::Check(
 	params["bandwidth"] = PString(arq.m_bandWidth.GetValue());
 	params["answer"] = arq.m_answerCall ? "1" : "0";
 	params["arq"] = "1";
+	params["CallId"] = AsString(arq.m_callIdentifier.m_guid);
 	
 	GkSQLResult::ResultRow result;	
 	if (!RunQuery(traceStr, m_sqlConn, m_callQuery, params, result, -1)) {
@@ -693,14 +694,14 @@ int SQLAuth::Check(
 		if (!s) {
 			PStringArray tokens(s.Tokenise("; \t", FALSE));
 			for (PINDEX i = 0; i < tokens.GetSize(); ++i) {
-				PIPSocket::Address addr;
+				PIPSocket::Address raddr;
 				WORD port = 0;
 					
-				if (GetTransportAddress(tokens[i], GK_DEF_ENDPOINT_SIGNAL_PORT, addr, port)
-						&& addr.IsValid() && port != 0) {
-					Route route("SQL", addr, port);
+				if (GetTransportAddress(tokens[i], GK_DEF_ENDPOINT_SIGNAL_PORT, raddr, port)
+						&& raddr.IsValid() && port != 0) {
+					Route route("SQL", raddr, port);
 					route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(
-						SocketToH225TransportAddr(addr, port)
+						SocketToH225TransportAddr(raddr, port)
 						);
 					authData.m_destinationRoutes.push_back(route);
 					PTRACE(5, traceStr << " - call redirected to the address " <<
@@ -826,6 +827,7 @@ int SQLAuth::Check(
 	params["Dialed-Number"] = GetDialedNumber(setup, authData);
 	params["answer"] = "0";
 	params["arq"] = "0";
+	params["CallId"] = AsString(setup.GetUUIEBody().m_callIdentifier.m_guid);
 
 	if (authData.m_call)
 		params["bandwidth"] = PString(authData.m_call->GetBandwidth());
@@ -876,14 +878,14 @@ int SQLAuth::Check(
 		if (!s) {
 			PStringArray tokens(s.Tokenise("; \t", FALSE));
 			for (PINDEX i = 0; i < tokens.GetSize(); ++i) {
-				PIPSocket::Address addr;
+				PIPSocket::Address raddr;
 				WORD port = 0;
 					
-				if (GetTransportAddress(tokens[i], GK_DEF_ENDPOINT_SIGNAL_PORT, addr, port)
-						&& addr.IsValid() && port != 0) {
-					Route route("SQL", addr, port);
+				if (GetTransportAddress(tokens[i], GK_DEF_ENDPOINT_SIGNAL_PORT, raddr, port)
+						&& raddr.IsValid() && port != 0) {
+					Route route("SQL", raddr, port);
 					route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(
-						SocketToH225TransportAddr(addr, port)
+						SocketToH225TransportAddr(raddr, port)
 						);
 					authData.m_destinationRoutes.push_back(route);
 					PTRACE(5, traceStr << " - call redirected to the address " <<
