@@ -23,6 +23,7 @@
 #include "stl_supp.h"
 #include "rwlock.h"
 #include "yasocket.h"
+#include "Toolkit.h"
 
 using std::mem_fun;
 using std::bind1st;
@@ -413,13 +414,21 @@ bool YaTCPSocket::Connect(const Address & iface, WORD localPort, const Address &
 	YaSelectList::large_fd_set fdset;
 	fdset.add(os_handle);
 	YaSelectList::large_fd_set exset = fdset;
-	struct timeval tval = { 6, 0 }; // TODO: read from config...
-	if (::select(os_handle + 1, 0, fdset, exset, &tval) > 0) {
+	struct timeval tval = { 6, 0 };
+	if ((r = ::select(os_handle + 1, 0, fdset, exset, &tval)) > 0) {
 		optval = -1;
 		::getsockopt(os_handle, SOL_SOCKET, SO_ERROR, &optval, &optlen);
 		if (optval == 0) // connected
 			return SetLinger();
 		errno = optval;
+	} else {
+		if (r == 0) {
+#ifdef _WIN32
+			errno = WSAETIMEDOUT;
+#else
+			errno = ETIMEDOUT;
+#endif
+		}
 	}
 	return ConvertOSError(-1);
 }
@@ -650,8 +659,8 @@ bool USocket::ErrorHandler(PSocket::ErrorGroup group)
 			if (group == PSocket::LastReadError) {
 				PTRACE(5, msg << " closed by remote");
 				CloseSocket();
-				break;
 			}
+			break;
 		default:
 			PTRACE(3, msg << " Error(" << group << "): " 
 				<< PSocket::GetErrorText(e) << " (" << e << ':'
@@ -721,8 +730,11 @@ SocketsReader::~SocketsReader()
 void SocketsReader::Stop()
 {
 	PWaitAndSignal lock(m_deletionPreventer);
-	ReadLock llock(m_listmutex);
+	
+	m_listmutex.StartWrite();
 	ForEachInContainer(m_sockets, mem_fun(&IPSocket::Close));
+	m_listmutex.EndWrite();
+	
 	RegularJob::Stop();
 }
 
@@ -775,7 +787,7 @@ bool SocketsReader::SelectSockets(SocketSelectList & slist)
 
 void SocketsReader::RemoveClosed(bool bDeleteImmediately)
 {
-	WriteLock lock(m_listmutex);
+	WriteLock listlock(m_listmutex);
 	iterator iter = partition(m_sockets.begin(), m_sockets.end(), mem_fun(&IPSocket::IsOpen));
 	if (ptrdiff_t rmsize = distance(iter, m_sockets.end())) {
 		if (bDeleteImmediately)
@@ -859,12 +871,26 @@ bool TCPListenSocket::IsTimeout(const PTime *now) const
 TCPServer::TCPServer()
 {
 	SetName("TCPSrv");
+	LoadConfig();
 	Execute();
 }
 
+void TCPServer::LoadConfig()
+{
+	cps_limit = GkConfig()->GetInteger("RoutedMode", "CpsLimit", 0);
+	if (cps_limit == 0)
+		PTRACE(4, "TCPSrv\tCpsLimit disabled");
+	else
+		PTRACE(4, "TCPSrv\tCpsLimit set to " << cps_limit);
+	if (cps_limit > 0) {
+		check_interval = GkConfig()->GetInteger("RoutedMode", "CpsCheckInterval", 5);
+		PTRACE(4, "TCPSrv\tCpsCheckInterval set to " << check_interval);
+	}
+}
+
 bool TCPServer::CloseListener(TCPListenSocket *socket)
 {
-	ReadLock lock(m_listmutex);
+	WriteLock lock(m_listmutex);
 	iterator iter = find(m_sockets.begin(), m_sockets.end(), socket);
 	if (iter != m_sockets.end()) {
 		PTRACE(6, GetName() << "\tListener " << (*iter)->GetName() << " closed");
@@ -877,6 +903,30 @@ bool TCPServer::CloseListener(TCPListenSocket *socket)
 void TCPServer::ReadSocket(IPSocket *socket)
 {
 	PTRACE(4, GetName() << "\tAccept request on " << socket->GetName());
+	// rate limiting
+	if (cps_limit > 0) {
+		time_t now = time(NULL);
+		// clear old values
+		one_sec.remove_if(bind2nd(not_equal_to<time_t>(), now));
+		many_sec.remove_if(bind2nd(less<time_t>(), now - check_interval));
+		PTRACE(4, GetName() << "\tcurrent cps=" << one_sec.size() << " calls in interval=" << many_sec.size());
+		if ((many_sec.size() > (cps_limit *  check_interval)) && (one_sec.size() > cps_limit)) {
+			// reject call
+			PTRACE(1, GetName() << "\tRate limit reached (max " << cps_limit << " cps) - rejecting call on " << socket->GetName());
+			int rej = ::accept(socket->GetHandle(), NULL, NULL);
+			::shutdown(rej, 2 /* SHUT_RDWR */ );
+#if defined(_WIN32)
+			::_close(rej);
+#else
+			::close(rej);
+#endif
+			return;
+		}
+		// add accepted calls to stats list
+		one_sec.push_back(now);
+		many_sec.push_back(now);
+	}
+
 	TCPListenSocket *listener = dynamic_cast<TCPListenSocket *>(socket);
 	ServerSocket *acceptor = listener->CreateAcceptor();
 	if (acceptor->Accept(*listener)) {
@@ -894,13 +944,12 @@ void TCPServer::CleanUp()
 	WriteLock lock(m_listmutex);
 	iterator iter = m_sockets.begin();
 	while (iter != m_sockets.end()) {
-		iterator i = iter++;
-		TCPListenSocket *listener = dynamic_cast<TCPListenSocket *>(*i);
+		TCPListenSocket *listener = dynamic_cast<TCPListenSocket *>(*iter);
 		if (listener && listener->IsTimeout(&now)) {
-			m_sockets.erase(i);
+			iter = m_sockets.erase(iter);
 			--m_socksize;
 			delete listener;
-			iter = m_sockets.begin();
 		}
+		else ++iter;
 	}
 }
